import os
import zipfile
import logging
import importlib.metadata
from functools import partial
from datetime import datetime, timedelta
from typing import List, Dict, Tuple, Literal, Generator

import cv2
import gradio as gr

from kotonebot.backend.context import task_registry, ContextStackVars
from kotonebot.config.manager import load_config, save_config
from kotonebot.tasks.common import (
    BaseConfig, APShopItems, CapsuleToysConfig, ClubRewardConfig, PurchaseConfig, ActivityFundsConfig,
    PresentsConfig, AssignmentConfig, ContestConfig, ProduceConfig,
    MissionRewardConfig, PIdol, DailyMoneyShopItems, ProduceAction,
    RecommendCardDetectionMode, TraceConfig, StartGameConfig, UpgradeSupportCardConfig,
    upgrade_config
)
from kotonebot.config.base_config import UserConfig, BackendConfig
from kotonebot.backend.bot import KotoneBot

# 初始化日志
os.makedirs('logs', exist_ok=True)
log_formatter = logging.Formatter('[%(asctime)s][%(levelname)s][%(name)s] %(message)s')
log_filename = datetime.now().strftime('logs/%y-%m-%d-%H-%M-%S.log')

console_handler = logging.StreamHandler()
console_handler.setFormatter(log_formatter)
file_handler = logging.FileHandler(log_filename, encoding='utf-8')
file_handler.setFormatter(log_formatter)

root_logger = logging.getLogger()
root_logger.setLevel(logging.INFO)
root_logger.addHandler(console_handler)
root_logger.addHandler(file_handler)

logging.getLogger("kotonebot").setLevel(logging.DEBUG)

# 升级配置
upgrade_msg = upgrade_config()

logger = logging.getLogger(__name__)

def _save_bug_report(
    path: str | None = None
) -> Generator[str, None, str]:
    """
    保存报告

    :param path: 保存的路径。若为 `None`，则保存到 `./reports/bug-{YY-MM-DD HH-MM-SS}.zip`。
    :return: 保存的路径
    """
    from kotonebot import device
    from kotonebot.backend.context import ContextStackVars
    
    # 确保目录存在
    os.makedirs('logs', exist_ok=True)
    os.makedirs('reports', exist_ok=True)

    error = ""
    if path is None:
        path = f'./reports/bug-{datetime.now().strftime("%y-%m-%d %H-%M-%S")}.zip'
    with zipfile.ZipFile(path, 'w', zipfile.ZIP_DEFLATED, compresslevel=9) as zipf:
        # 打包截图
        yield "### 打包上次截图..."
        try:
            stack = ContextStackVars.current()
            screenshot = None
            if stack is not None:
                screenshot = stack._screenshot
                if screenshot is not None:
                    img = cv2.imencode('.png', screenshot)[1].tobytes()
                    zipf.writestr('last_screenshot.png', img)
            if screenshot is None:
                error += "无上次截图数据\n"
        except Exception as e:
            error += f"保存上次截图失败：{str(e)}\n"

        # 打包当前截图
        yield "### 打包当前截图..."
        try:
            screenshot = device.screenshot()
            img = cv2.imencode('.png', screenshot)[1].tobytes()
            zipf.writestr('current_screenshot.png', img)
        except Exception as e:
            error += f"保存当前截图失败：{str(e)}\n"

        # 打包配置文件
        yield "### 打包配置文件..."
        try:
            with open('config.json', 'r', encoding='utf-8') as f:
                zipf.writestr('config.json', f.read())
        except Exception as e:
            error += f"保存配置文件失败：{str(e)}\n"

        # 打包 logs 文件夹
        if os.path.exists('logs'):
            for root, dirs, files in os.walk('logs'):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.join('logs', os.path.relpath(file_path, 'logs'))
                    zipf.write(file_path, arcname)
                    yield f"### 打包 log 文件：{arcname}"

        # 写出版本号
        zipf.writestr('version.txt', version)
    
    # 上传报告
    from .file_host.sensio import upload
    yield "### 上传报告..."
    url = ''
    try:
        url = upload(path)
    except Exception as e:
        yield f"### 上传报告失败：{str(e)}\n\n"
        return ''

    final_msg = f"### 报告导出成功：{url}\n\n"
    expire_time = datetime.now() + timedelta(days=7)
    if error:
        final_msg += f"### 但发生了以下错误\n\n"
        final_msg += '\n* '.join(error.strip().split('\n'))
    final_msg += '\n'
    final_msg += f"### 此链接将于 {expire_time.strftime('%Y-%m-%d %H:%M:%S')}（7 天后）过期\n\n"
    final_msg += '### 复制以上文本并反馈给开发者'
    yield final_msg
    return path

version = importlib.metadata.version('ksaa')
class KotoneBotUI:
    def __init__(self) -> None:
        self.is_running: bool = False
        self.kaa: KotoneBot = KotoneBot(module='kotonebot.tasks', config_type=BaseConfig)
        self._load_config()
        self._setup_kaa()
        
    def _setup_kaa(self) -> None:
        from kotonebot.backend.debug.vars import debug, clear_saved
        logger.info('Version: %s', version)

        self.kaa.initialize()
        if self.current_config.keep_screenshots:
            debug.auto_save_to_folder = 'dumps'
            debug.enabled = True
            clear_saved()
        else:
            debug.auto_save_to_folder = None
            debug.enabled = False

    def export_dumps(self) -> str:
        """导出 dumps 文件夹为 zip 文件"""
        if not os.path.exists('dumps'):
            return "dumps 文件夹不存在"
        
        timestamp = datetime.now().strftime('%y-%m-%d-%H-%M-%S')
        zip_filename = f'dumps-{timestamp}.zip'
        
        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED, compresslevel=9) as zipf:
            for root, dirs, files in os.walk('dumps'):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, 'dumps')
                    zipf.write(file_path, arcname)
        
        return f"已导出到 {zip_filename}"

    def export_logs(self) -> str:
        """导出 logs 文件夹为 zip 文件"""
        if not os.path.exists('logs'):
            return "logs 文件夹不存在"
        
        timestamp = datetime.now().strftime('%y-%m-%d-%H-%M-%S')
        zip_filename = f'logs-{timestamp}.zip'
        
        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED, compresslevel=9) as zipf:
            for root, dirs, files in os.walk('logs'):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, 'logs')
                    zipf.write(file_path, arcname)
        
        return f"已导出到 {zip_filename}"

    def get_button_status(self) -> str:
        if not hasattr(self, 'run_status'):
            return "启动"
        
        if not self.run_status.running:
            self.is_running = False
            return "启动"
        return "停止"

    def update_task_status(self) -> List[List[str]]:
        status_list: List[List[str]] = []
        if not hasattr(self, 'run_status'):
            for task_name, task in task_registry.items():
                status_list.append([task.name, "等待中"])
            return status_list
        
        for task_status in self.run_status.tasks:
            status_text = {
                'pending': '等待中',
                'running': '运行中',
                'finished': '已完成',
                'error': '出错',
                'cancelled': '已取消'
            }.get(task_status.status, '未知')
            status_list.append([task_status.task.name, status_text])
        return status_list

    def toggle_run(self) -> Tuple[str, List[List[str]]]:
        if not self.is_running:
            return self.start_run()
        return self.stop_run()
    
    def start_run(self) -> Tuple[str, List[List[str]]]:
        self.is_running = True
        self.run_status = self.kaa.start_all()
        return "停止", self.update_task_status()

    def stop_run(self) -> Tuple[str, List[List[str]]]:
        self.is_running = False
        if self.kaa:
            self.run_status.interrupt()
        return "启动", self.update_task_status()

    def save_settings(
        self,
        adb_ip: str,
        adb_port: int,
        screenshot_method: Literal['adb', 'adb_raw', 'uiautomator2', 'windows'],
        keep_screenshots: bool,
        check_emulator: bool,
        emulator_path: str,
        adb_emulator_name: str,
        trace_recommend_card_detection: bool,
        purchase_enabled: bool,
        money_enabled: bool,
        ap_enabled: bool,
        ap_items: List[str],
        money_items: List[DailyMoneyShopItems],
        activity_funds_enabled: bool,
        presents_enabled: bool,
        assignment_enabled: bool,
        mini_live_reassign: bool,
        mini_live_duration: Literal[4, 6, 12],
        online_live_reassign: bool,
        online_live_duration: Literal[4, 6, 12],
        contest_enabled: bool,
        select_which_contestant: Literal[1, 2, 3],
        produce_enabled: bool,
        produce_mode: Literal["regular"],
        produce_count: int,
        produce_idols: List[str],
        memory_sets: List[str],
        auto_set_memory: bool,
        auto_set_support: bool,
        use_pt_boost: bool,
        use_note_boost: bool,
        follow_producer: bool,
        self_study_lesson: Literal['dance', 'visual', 'vocal'],
        prefer_lesson_ap: bool,
        actions_order: List[str],
        recommend_card_detection_mode: str,
        mission_reward_enabled: bool,
        # club reward
        club_reward_enabled: bool,
        selected_note: DailyMoneyShopItems,
        # upgrade support card
        upgrade_support_card_enabled: bool,
        # capsule toys
        capsule_toys_enabled: bool,
		friend_capsule_toys_count: int,
		sense_capsule_toys_count: int,
		logic_capsule_toys_count: int,
		anomaly_capsule_toys_count: int,
        # start game
        start_game_enabled: bool,
        start_through_kuyo: bool,
        game_package_name: str,
        kuyo_package_name: str,
    ) -> str:
        ap_items_enum: List[Literal[0, 1, 2, 3]] = []
        ap_items_map: Dict[str, APShopItems] = {
            "支援强化点数提升": APShopItems.PRODUCE_PT_UP,
            "笔记数提升": APShopItems.PRODUCE_NOTE_UP,
            "重新挑战券": APShopItems.RECHALLENGE,
            "回忆再生成券": APShopItems.REGENERATE_MEMORY
        }
        for item in ap_items:
            if item in ap_items_map:
                ap_items_enum.append(ap_items_map[item].value)  # type: ignore
        
        self.current_config.backend.adb_ip = adb_ip
        self.current_config.backend.adb_port = adb_port
        self.current_config.backend.adb_emulator_name = adb_emulator_name
        self.current_config.backend.screenshot_impl = screenshot_method
        self.current_config.keep_screenshots = keep_screenshots
        self.current_config.backend.check_emulator = check_emulator
        self.current_config.backend.emulator_path = emulator_path
        
        options = BaseConfig(
            purchase=PurchaseConfig(
                enabled=purchase_enabled,
                money_enabled=money_enabled,
                money_items=money_items,
                ap_enabled=ap_enabled,
                ap_items=ap_items_enum
            ),
            activity_funds=ActivityFundsConfig(
                enabled=activity_funds_enabled
            ),
            presents=PresentsConfig(
                enabled=presents_enabled
            ),
            assignment=AssignmentConfig(
                enabled=assignment_enabled,
                mini_live_reassign_enabled=mini_live_reassign,
                mini_live_duration=mini_live_duration,
                online_live_reassign_enabled=online_live_reassign,
                online_live_duration=online_live_duration
            ),
            contest=ContestConfig(
                enabled=contest_enabled,
                select_which_contestant=select_which_contestant
            ),
            produce=ProduceConfig(
                enabled=produce_enabled,
                mode=produce_mode,
                produce_count=produce_count,
                idols=[PIdol[idol] for idol in produce_idols],
                memory_sets=[int(i) for i in memory_sets],
                auto_set_memory=auto_set_memory,
                auto_set_support_card=auto_set_support,
                use_pt_boost=use_pt_boost,
                use_note_boost=use_note_boost,
                follow_producer=follow_producer,
                self_study_lesson=self_study_lesson,
                prefer_lesson_ap=prefer_lesson_ap,
                actions_order=[ProduceAction(action) for action in actions_order],
                recommend_card_detection_mode=RecommendCardDetectionMode(recommend_card_detection_mode)
            ),
            mission_reward=MissionRewardConfig(
                enabled=mission_reward_enabled
            ),
            club_reward=ClubRewardConfig(
                enabled=club_reward_enabled,
                selected_note=selected_note
            ),
            upgrade_support_card=UpgradeSupportCardConfig(
                enabled=upgrade_support_card_enabled
            ),
            capsule_toys=CapsuleToysConfig(
                enabled=capsule_toys_enabled,
                friend_capsule_toys_count=friend_capsule_toys_count,
                sense_capsule_toys_count=sense_capsule_toys_count,
                logic_capsule_toys_count=logic_capsule_toys_count,
                anomaly_capsule_toys_count=anomaly_capsule_toys_count
            ),
            start_game=StartGameConfig(
                enabled=start_game_enabled,
                start_through_kuyo=start_through_kuyo,
                game_package_name=game_package_name,
                kuyo_package_name=kuyo_package_name
            ),
            trace=TraceConfig(
                recommend_card_detection=trace_recommend_card_detection
            )
        )
        
        self.current_config.options = options
        
        try:
            save_config(self.config, "config.json")
            gr.Success("设置已保存，请重启程序！")
            return ""
        except Exception as e:
            gr.Warning(f"保存设置失败：{str(e)}")
            return ""

    def _create_status_tab(self) -> None:
        with gr.Tab("状态"):
            gr.Markdown("## 状态")
            
            with gr.Row():
                run_btn = gr.Button("启动", scale=1)
            if upgrade_msg:
                gr.Markdown('### 配置升级报告')
                gr.Markdown(upgrade_msg)
            gr.Markdown('脚本报错或者卡住？点击"日志"选项卡中的"一键导出报告"可以快速反馈！')
            
            task_status = gr.Dataframe(
                headers=["任务", "状态"],
                value=self.update_task_status(),
                label="任务状态"
            )
            
            def on_run_click(evt: gr.EventData) -> Tuple[str, List[List[str]]]:
                return self.toggle_run()
            
            run_btn.click(
                fn=on_run_click,
                outputs=[run_btn, task_status]
            )

            # 添加定时器，分别更新按钮状态和任务状态
            gr.Timer(1.0).tick(
                fn=self.get_button_status,
                outputs=[run_btn]
            )
            gr.Timer(1.0).tick(
                fn=self.update_task_status,
                outputs=[task_status]
            )

    def _create_task_tab(self) -> None:
        with gr.Tab("任务"):
            gr.Markdown("## 执行任务")
            
            # 创建任务选择下拉框
            task_choices = [task.name for task in task_registry.values()]
            task_dropdown = gr.Dropdown(
                choices=task_choices,
                label="选择要执行的任务",
                info="选择一个要单独执行的任务",
                type="value",
                value=None
            )
            
            # 创建执行按钮
            execute_btn = gr.Button("执行任务")
            task_result = gr.Markdown("")
            
            # TODO: 实现任务执行逻辑
            def execute_single_task(task_name: str) -> str:
                if not task_name:
                    gr.Warning("请先选择一个任务")
                    return ""
                task = None
                for name, task in task_registry.items():
                    if name == task_name:
                        task = task
                        break
                if task is None:
                    gr.Warning(f"任务 {task_name} 未找到")
                    return ""
                gr.Info(f"任务 {task_name} 开始执行。执行结束前，请勿重复点击执行。")
                self.kaa.run([task])
                gr.Success(f"任务 {task_name} 执行完毕")
                return ""
            
            execute_btn.click(
                fn=execute_single_task,
                inputs=[task_dropdown],
                outputs=[task_result]
            )

    def _create_emulator_settings(self) -> Tuple[gr.Textbox, gr.Number, gr.Dropdown, gr.Checkbox, gr.Checkbox, gr.Textbox, gr.Textbox]:
        with gr.Column():
            gr.Markdown("### 模拟器设置")
            adb_ip = gr.Textbox(
                value=self.current_config.backend.adb_ip,
                label="ADB IP 地址",
                info=BackendConfig.model_fields['adb_ip'].description,
                interactive=True
            )
            adb_port = gr.Number(
                value=self.current_config.backend.adb_port,
                label="ADB 端口",
                info=BackendConfig.model_fields['adb_port'].description,
                minimum=1,
                maximum=65535,
                step=1,
                interactive=True
            )
            screenshot_impl = gr.Dropdown(
                choices=['adb', 'adb_raw', 'uiautomator2', 'windows'],
                value=self.current_config.backend.screenshot_impl,
                label="截图方法",
                info=BackendConfig.model_fields['screenshot_impl'].description,
                interactive=True
            )
            keep_screenshots = gr.Checkbox(
                label="保留截图数据",
                value=self.current_config.keep_screenshots,
                info=UserConfig.model_fields['keep_screenshots'].description,
                interactive=True
            )
            check_emulator = gr.Checkbox(
                label="检查并启动模拟器",
                value=self.current_config.backend.check_emulator,
                info=BackendConfig.model_fields['check_emulator'].description,
                interactive=True
            )
            with gr.Group(visible=self.current_config.backend.check_emulator) as check_emulator_group:
                emulator_path = gr.Textbox(
                    value=self.current_config.backend.emulator_path,
                    label="模拟器 exe 文件路径",
                    info=BackendConfig.model_fields['emulator_path'].description,
                    interactive=True
                )
                adb_emulator_name = gr.Textbox(
                    value=self.current_config.backend.adb_emulator_name,
                    label="ADB 模拟器名称",
                    info=BackendConfig.model_fields['adb_emulator_name'].description,
                    interactive=True
                )
            check_emulator.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[check_emulator],
                outputs=[check_emulator_group]
            )
        return adb_ip, adb_port, screenshot_impl, keep_screenshots, check_emulator, emulator_path, adb_emulator_name

    def _create_purchase_settings(self) -> Tuple[gr.Checkbox, gr.Checkbox, gr.Checkbox, gr.Dropdown, gr.Dropdown]:
        with gr.Column():
            gr.Markdown("### 商店购买设置")
            purchase_enabled = gr.Checkbox(
                label="启用商店购买",
                value=self.current_config.options.purchase.enabled,
                info=PurchaseConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.purchase.enabled) as purchase_group:
                money_enabled = gr.Checkbox(
                    label="启用金币购买",
                    value=self.current_config.options.purchase.money_enabled,
                    info=PurchaseConfig.model_fields['money_enabled'].description
                )
                
                # 添加金币商店商品选择
                money_items = gr.Dropdown(
                    multiselect=True,
                    choices=list(DailyMoneyShopItems.all()),
                    value=self.current_config.options.purchase.money_items,
                    label="金币商店购买物品",
                    info=PurchaseConfig.model_fields['money_items'].description
                )
                
                ap_enabled = gr.Checkbox(
                    label="启用AP购买",
                    value=self.current_config.options.purchase.ap_enabled,
                    info=PurchaseConfig.model_fields['ap_enabled'].description
                )
                
                # 转换枚举值为显示文本
                selected_items: List[str] = []
                ap_items_map = {
                    APShopItems.PRODUCE_PT_UP: "支援强化点数提升",
                    APShopItems.PRODUCE_NOTE_UP: "笔记数提升",
                    APShopItems.RECHALLENGE: "重新挑战券",
                    APShopItems.REGENERATE_MEMORY: "回忆再生成券"
                }
                for item_value in self.current_config.options.purchase.ap_items:
                    item_enum = APShopItems(item_value)
                    if item_enum in ap_items_map:
                        selected_items.append(ap_items_map[item_enum])
                
                ap_items = gr.Dropdown(
                    multiselect=True,
                    choices=list(ap_items_map.values()),
                    value=selected_items,
                    label="AP商店购买物品",
                    info=PurchaseConfig.model_fields['ap_items'].description
                )
            
            purchase_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[purchase_enabled],
                outputs=[purchase_group]
            )
        return purchase_enabled, money_enabled, ap_enabled, ap_items, money_items

    def _create_work_settings(self) -> Tuple[gr.Checkbox, gr.Checkbox, gr.Dropdown, gr.Checkbox, gr.Dropdown]:
        with gr.Column():
            gr.Markdown("### 工作设置")
            assignment_enabled = gr.Checkbox(
                label="启用工作",
                value=self.current_config.options.assignment.enabled,
                info=AssignmentConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.assignment.enabled) as work_group:
                with gr.Row():
                    with gr.Column():
                        mini_live_reassign = gr.Checkbox(
                            label="启用重新分配 MiniLive",
                            value=self.current_config.options.assignment.mini_live_reassign_enabled,
                            info=AssignmentConfig.model_fields['mini_live_reassign_enabled'].description
                        )
                        mini_live_duration = gr.Dropdown(
                            choices=[4, 6, 12],
                            value=self.current_config.options.assignment.mini_live_duration,
                            label="MiniLive 工作时长",
                            interactive=True,
                            info=AssignmentConfig.model_fields['mini_live_duration'].description
                        )
                    with gr.Column():
                        online_live_reassign = gr.Checkbox(
                            label="启用重新分配 OnlineLive",
                            value=self.current_config.options.assignment.online_live_reassign_enabled,
                            info=AssignmentConfig.model_fields['online_live_reassign_enabled'].description
                        )
                        online_live_duration = gr.Dropdown(
                            choices=[4, 6, 12],
                            value=self.current_config.options.assignment.online_live_duration,
                            label="OnlineLive 工作时长",
                            interactive=True,
                            info=AssignmentConfig.model_fields['online_live_duration'].description
                        )
            
            assignment_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[assignment_enabled],
                outputs=[work_group]
            )
        return assignment_enabled, mini_live_reassign, mini_live_duration, online_live_reassign, online_live_duration
    
    def _create_contest_settings(self) -> Tuple[gr.Checkbox, gr.Dropdown]:
        with gr.Column():
            gr.Markdown("### 竞赛设置")
            contest_enabled = gr.Checkbox(
                label="启用竞赛",
                value=self.current_config.options.contest.enabled,
                info=ContestConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.contest.enabled) as contest_group:
                select_which_contestant = gr.Dropdown(
                    choices=[1, 2, 3],
                    value=self.current_config.options.contest.select_which_contestant,
                    label="选择第几位竞赛目标",
                    interactive=True,
                    info=ContestConfig.model_fields['select_which_contestant'].description
                )
            contest_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[contest_enabled],
                outputs=[contest_group]
            )
        return contest_enabled, select_which_contestant

    def _create_produce_settings(self):
        with gr.Column():
            gr.Markdown("### 培育设置")
            produce_enabled = gr.Checkbox(
                label="启用培育",
                value=self.current_config.options.produce.enabled,
                info=ProduceConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.produce.enabled) as produce_group:
                produce_mode = gr.Dropdown(
                    choices=["regular", "pro"],
                    value=self.current_config.options.produce.mode,
                    label="培育模式",
                    info=ProduceConfig.model_fields['mode'].description
                )
                produce_count = gr.Number(
                    minimum=1,
                    value=self.current_config.options.produce.produce_count,
                    label="培育次数",
                    interactive=True,
                    info=ProduceConfig.model_fields['produce_count'].description
                )
                # 添加偶像选择
                idol_choices = [idol.name for idol in PIdol]
                selected_idols = [idol.name for idol in self.current_config.options.produce.idols]
                produce_idols = gr.Dropdown(
                    choices=idol_choices,
                    value=selected_idols,
                    label="选择要培育的偶像",
                    multiselect=True,
                    interactive=True,
                    info=ProduceConfig.model_fields['idols'].description
                )
                has_kotone = any("藤田ことね" in idol for idol in selected_idols)
                is_strict_mode = self.current_config.options.produce.recommend_card_detection_mode == RecommendCardDetectionMode.STRICT
                kotone_warning = gr.Markdown(
                    visible=has_kotone and not is_strict_mode,
                    value="使用「藤田ことね」进行培育时，确保将「推荐卡检测模式」设置为「严格模式」"
                )
                auto_set_memory = gr.Checkbox(
                    label="自动编成回忆",
                    value=self.current_config.options.produce.auto_set_memory,
                    info=ProduceConfig.model_fields['auto_set_memory'].description
                )
                with gr.Group(visible=not self.current_config.options.produce.auto_set_memory) as memory_sets_group:
                    memory_sets = gr.Dropdown(
                        choices=[str(i) for i in range(1, 11)],  # 假设最多10个编成位
                        value=[str(i) for i in self.current_config.options.produce.memory_sets],
                        label="回忆编成编号",
                        multiselect=True,
                        interactive=True,
                        info=ProduceConfig.model_fields['memory_sets'].description
                    )
                
                # 添加偶像选择变化时的回调
                def update_kotone_warning(selected_idols, recommend_card_detection_mode):
                    has_kotone = any("藤田ことね" in idol for idol in selected_idols)
                    is_strict_mode = recommend_card_detection_mode == RecommendCardDetectionMode.STRICT.value
                    return gr.Markdown(visible=has_kotone and not is_strict_mode)
                
                auto_set_support = gr.Checkbox(
                    label="自动编成支援卡",
                    value=self.current_config.options.produce.auto_set_support_card,
                    info=ProduceConfig.model_fields['auto_set_support_card'].description
                )
                use_pt_boost = gr.Checkbox(
                    label="使用支援强化 Pt 提升",
                    value=self.current_config.options.produce.use_pt_boost,
                    info=ProduceConfig.model_fields['use_pt_boost'].description
                )
                use_note_boost = gr.Checkbox(
                    label="使用笔记数提升",
                    value=self.current_config.options.produce.use_note_boost,
                    info=ProduceConfig.model_fields['use_note_boost'].description
                )
                follow_producer = gr.Checkbox(
                    label="关注租借了支援卡的制作人",
                    value=self.current_config.options.produce.follow_producer,
                    info=ProduceConfig.model_fields['follow_producer'].description
                )
                self_study_lesson = gr.Dropdown(
                    choices=['dance', 'visual', 'vocal'],
                    value=self.current_config.options.produce.self_study_lesson,
                    label='文化课自习时选项',
                    info='选择自习课类型'
                )
                prefer_lesson_ap = gr.Checkbox(
                    label="SP 课程优先",
                    value=self.current_config.options.produce.prefer_lesson_ap,
                    info=ProduceConfig.model_fields['prefer_lesson_ap'].description
                )
                actions_order = gr.Dropdown(
                    choices=[(action.display_name, action.value) for action in ProduceAction],
                    value=[action.value for action in self.current_config.options.produce.actions_order],
                    label="行动优先级",
                    info="设置每周行动的优先级顺序",
                    multiselect=True
                )

                # 添加推荐卡检测模式设置
                recommend_card_detection_mode = gr.Dropdown(
                    choices=[
                        (RecommendCardDetectionMode.NORMAL.display_name, RecommendCardDetectionMode.NORMAL.value),
                        (RecommendCardDetectionMode.STRICT.display_name, RecommendCardDetectionMode.STRICT.value)
                    ],
                    value=self.current_config.options.produce.recommend_card_detection_mode.value,
                    label="推荐卡检测模式",
                    info=ProduceConfig.model_fields['recommend_card_detection_mode'].description
                )
                recommend_card_detection_mode.change(
                    fn=update_kotone_warning,
                    inputs=[produce_idols, recommend_card_detection_mode],
                    outputs=kotone_warning
                )
                produce_idols.change(
                    fn=update_kotone_warning,
                    inputs=[produce_idols, recommend_card_detection_mode],
                    outputs=kotone_warning
                )

            produce_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[produce_enabled],
                outputs=[produce_group]
            )
            
            auto_set_memory.change(
                fn=lambda x: gr.Group(visible=not x),
                inputs=[auto_set_memory],
                outputs=[memory_sets_group]
            )
        return produce_enabled, produce_mode, produce_count, produce_idols, memory_sets, auto_set_memory, auto_set_support, use_pt_boost, use_note_boost, follow_producer, self_study_lesson, prefer_lesson_ap, actions_order, recommend_card_detection_mode
    
    def _create_club_reward_settings(self) -> Tuple[gr.Checkbox, gr.Dropdown]:
        with gr.Column():
            gr.Markdown("### 社团奖励设置")
            club_reward_enabled = gr.Checkbox(
                label="启用社团奖励",
                value=self.current_config.options.club_reward.enabled,
                info=ClubRewardConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.club_reward.enabled) as club_reward_group:
                selected_note = gr.Dropdown(
                    choices=list(DailyMoneyShopItems.note_items()),
                    value=self.current_config.options.club_reward.selected_note,
                    label="想在社团奖励中获取到的笔记",
                    interactive=True,
                    info=ClubRewardConfig.model_fields['selected_note'].description
                )
            club_reward_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[club_reward_enabled],
                outputs=[club_reward_group]
            )
        return club_reward_enabled, selected_note

    def _create_capsule_toys_settings(self) -> Tuple[gr.Checkbox, gr.Number, gr.Number, gr.Number, gr.Number]:
        with gr.Column():
            gr.Markdown("### 扭蛋设置")
            capsule_toys_enabled = gr.Checkbox(
                label="是否启用自动扭蛋机",
                value=self.current_config.options.capsule_toys.enabled,
                info=CapsuleToysConfig.model_fields['enabled'].description
            )
            min_value = 0
            max_value = 10
            with gr.Group(visible=self.current_config.options.capsule_toys.enabled) as capsule_toys_group:
                friend_capsule_toys_count = gr.Number(
                    value=self.current_config.options.capsule_toys.friend_capsule_toys_count,
                    label="好友扭蛋机的扭蛋次数",
                    info=CapsuleToysConfig.model_fields['friend_capsule_toys_count'].description,
                    minimum=0,
                    maximum=5
                )
                sense_capsule_toys_count = gr.Number(
                    value=self.current_config.options.capsule_toys.sense_capsule_toys_count,
                    label="感性扭蛋机的扭蛋次数",
                    info=CapsuleToysConfig.model_fields['sense_capsule_toys_count'].description,
                    minimum=0,
                    maximum=5
                )
                logic_capsule_toys_count = gr.Number(
                    value=self.current_config.options.capsule_toys.logic_capsule_toys_count,
                    label="逻辑扭蛋机的扭蛋次数",
                    info=CapsuleToysConfig.model_fields['logic_capsule_toys_count'].description,
                    minimum=0,
                    maximum=5
                )
                anomaly_capsule_toys_count = gr.Number(
                    value=self.current_config.options.capsule_toys.anomaly_capsule_toys_count,
                    label="非凡扭蛋机的扭蛋次数",
                    info=CapsuleToysConfig.model_fields['anomaly_capsule_toys_count'].description,
                    minimum=0,
                    maximum=5
                )
            capsule_toys_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[capsule_toys_enabled],
                outputs=[capsule_toys_group]
            )
        return capsule_toys_enabled, friend_capsule_toys_count, sense_capsule_toys_count, logic_capsule_toys_count, anomaly_capsule_toys_count


    def _create_start_game_settings(self) -> Tuple[gr.Checkbox, gr.Checkbox, gr.Textbox, gr.Textbox]:
        with gr.Column():
            gr.Markdown("### 启动游戏设置")
            start_game_enabled = gr.Checkbox(
                label="是否启用 自动启动游戏",
                value=self.current_config.options.start_game.enabled,
                info=StartGameConfig.model_fields['enabled'].description
            )
            with gr.Group(visible=self.current_config.options.start_game.enabled) as start_game_group:
                start_through_kuyo = gr.Checkbox(
                    label="是否通过Kuyo启动游戏",
                    value=self.current_config.options.start_game.start_through_kuyo,
                    info=StartGameConfig.model_fields['start_through_kuyo'].description
                )
                game_package_name = gr.Textbox(
                    value=self.current_config.options.start_game.game_package_name,
                    label="游戏包名",
                    info=StartGameConfig.model_fields['game_package_name'].description
                )
                kuyo_package_name = gr.Textbox(
                    value=self.current_config.options.start_game.kuyo_package_name,
                    label="Kuyo包名",
                    info=StartGameConfig.model_fields['kuyo_package_name'].description
                )
            start_game_enabled.change(
                fn=lambda x: gr.Group(visible=x),
                inputs=[start_game_enabled],
                outputs=[start_game_group]
            )
        return start_game_enabled, start_through_kuyo, game_package_name, kuyo_package_name

    def _create_settings_tab(self) -> None:
        with gr.Tab("设置"):
            gr.Markdown("## 设置")
            
            # 模拟器设置
            emulator_settings = self._create_emulator_settings()
            
            # 商店购买设置
            purchase_settings = self._create_purchase_settings()
            
            # 活动费设置
            with gr.Column():
                gr.Markdown("### 活动费设置")
                activity_funds = gr.Checkbox(
                    label="启用收取活动费",
                    value=self.current_config.options.activity_funds.enabled,
                    info=ActivityFundsConfig.model_fields['enabled'].description
                )
            
            # 礼物设置
            with gr.Column():
                gr.Markdown("### 礼物设置")
                presents = gr.Checkbox(
                    label="启用收取礼物",
                    value=self.current_config.options.presents.enabled,
                    info=PresentsConfig.model_fields['enabled'].description
                )
            
            # 工作设置
            work_settings = self._create_work_settings()
            
            # 竞赛设置
            contest_settings = self._create_contest_settings()
            
            # 培育设置
            produce_settings = self._create_produce_settings()
            
            # 任务奖励设置
            with gr.Column():
                gr.Markdown("### 任务奖励设置")
                mission_reward = gr.Checkbox(
                    label="启用领取任务奖励",
                    value=self.current_config.options.mission_reward.enabled,
                    info=MissionRewardConfig.model_fields['enabled'].description
                )
            
            # 社团奖励设置
            club_reward_settings = self._create_club_reward_settings()

            # 升级支援卡设置
            with gr.Column():
                gr.Markdown("### 升级支援卡设置")
                upgrade_support_card_enabled = gr.Checkbox(
                    label="启用升级支援卡",
                    value=self.current_config.options.upgrade_support_card.enabled,
                    info=UpgradeSupportCardConfig.model_fields['enabled'].description
                )
            
            capsule_toys_settings = self._create_capsule_toys_settings()
            
            # 跟踪设置
            with gr.Column():
                gr.Markdown("### 跟踪设置")
                gr.Markdown("跟踪功能会记录指定模块的运行情况，并保存截图。")
                trace_recommend_card_detection = gr.Checkbox(
                    label="跟踪推荐卡检测",
                    value=self.current_config.options.trace.recommend_card_detection,
                    info=TraceConfig.model_fields['recommend_card_detection'].description,
                    interactive=True
                )

            # 启动游戏设置
            start_game_settings = self._create_start_game_settings()
            
            save_btn = gr.Button("保存设置")
            result = gr.Markdown()
            
            # 收集所有设置组件
            all_settings = [
                *emulator_settings,
                trace_recommend_card_detection,
                *purchase_settings,
                activity_funds,
                presents,
                *work_settings,
                *contest_settings,
                *produce_settings,
                mission_reward,
                *club_reward_settings,
                upgrade_support_card_enabled,
                *capsule_toys_settings,
                *start_game_settings
            ]
            
            save_btn.click(
                fn=self.save_settings,
                inputs=all_settings,
                outputs=result
            )

    def _create_log_tab(self) -> None:
        with gr.Tab("日志"):
            gr.Markdown("## 日志")
            
            with gr.Column():
                with gr.Row():
                    export_dumps_btn = gr.Button("导出 dump")
                    export_logs_btn = gr.Button("导出日志")
                with gr.Row():
                    save_report_btn = gr.Button("一键导出报告")
                result_text = gr.Markdown("等待操作\n\n\n")
            
            export_dumps_btn.click(
                fn=self.export_dumps,
                outputs=[result_text]
            )
            export_logs_btn.click(
                fn=self.export_logs,
                outputs=[result_text]
            )
            save_report_btn.click(
                fn=partial(_save_bug_report),
                outputs=[result_text]
            )

    def _create_whats_new_tab(self) -> None:
        """创建更新日志标签页，并显示最新版本更新内容"""
        with gr.Tab("更新日志"):
            from kotonebot.tasks.metadata import WHATS_NEW
            gr.Markdown(WHATS_NEW)

    def _create_screen_tab(self) -> None:
        with gr.Tab("画面"):
            gr.Markdown("## 当前设备画面")
            WIDTH = 720 // 3
            HEIGHT = 1280 // 3
            last_update_text = gr.Markdown("上次更新时间：无数据")
            screenshot_display = gr.Image(type="numpy", width=WIDTH, height=HEIGHT)

            def update_screenshot():
                ctx = ContextStackVars.current()
                if ctx is None:
                    return [None, last_update_text.value]
                screenshot = ctx._screenshot
                if screenshot is None:
                    return [None, last_update_text.value]
                screenshot = cv2.cvtColor(screenshot, cv2.COLOR_BGR2RGB)
                return screenshot, f"上次更新时间：{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"

            gr.Timer(0.3).tick(
                fn=update_screenshot,
                outputs=[screenshot_display, last_update_text]
            )

    def _load_config(self) -> None:
        # 加载配置文件
        config_path = "config.json"
        self.config = load_config(config_path, type=BaseConfig, use_default_if_not_found=True)
        if not self.config.user_configs:
            # 如果没有用户配置，创建一个默认配置
            default_config = UserConfig[BaseConfig](
                name="默认配置",
                category="default",
                description="默认配置",
                backend=BackendConfig(),
                options=BaseConfig()
            )
            self.config.user_configs.append(default_config)
        self.current_config = self.config.user_configs[0]

    def create_ui(self) -> gr.Blocks:
        with gr.Blocks(title="琴音小助手", css="#container { max-width: 800px; margin: auto; padding: 20px; }") as app:
            with gr.Column(elem_id="container"):
                gr.Markdown(f"# 琴音小助手 v{version}")
                
                with gr.Tabs():
                    self._create_status_tab()
                    self._create_task_tab()
                    self._create_settings_tab()
                    self._create_log_tab()
                    self._create_whats_new_tab()
                    self._create_screen_tab()
            
        return app

def main() -> None:
    ui = KotoneBotUI()
    app = ui.create_ui()
    app.launch(inbrowser=True, show_error=True, server_name="0.0.0.0")

if __name__ == "__main__":
    main()



import colorsys
from typing import Literal
from dataclasses import dataclass

import numpy as np
import cv2
from cv2.typing import MatLike

from .core import unify_image
from .debug import result as debug_result, debug, color as debug_color
from ..util import Rect

RgbColorTuple = tuple[int, int, int]
RgbColorStr = str
RgbColor = RgbColorTuple | RgbColorStr
"""颜色。三元组 `(r, g, b)` 或十六进制颜色字符串 `#RRGGBB`"""

HsvColor = tuple[int, int, int]
"""
HSV颜色。三元组 `(h, s, v)`。
"""

@dataclass
class FindColorPointResult:
    position: tuple[int, int]
    confidence: float
    target_color: RgbColor

def hsv_web2cv(h: int, s: int, v: int) -> 'HsvColor':
    """
    将 HSV 颜色从 Web 格式转换为 OpenCV 格式。
    
    :param h: 色相，范围 [0, 360]
    :param s: 饱和度，范围 [0, 100]
    :param v: 亮度，范围 [0, 100]
    :return: OpenCV 格式 HSV 颜色。三元组 `(h, s, v)`，范围分别为 (0-180, 0-255, 0-255)。
    """
    h = round(h / 2)  # web 的色相范围是 0-360，转为 0-180
    s = round(s / 100 * 255)  # web 的饱和度范围是 0-100，转为 0-255
    v = round(v / 100 * 255)  # web 的亮度范围是 0-100，转为 0-255
    return (h, s, v)

def hsv_cv2web(h: int, s: int, v: int) -> 'HsvColor':
    """
    将 HSV 颜色从 OpenCV 格式转换为 Web 格式。
    
    :param h: 色相，范围 [0, 180]
    :param s: 饱和度，范围 [0, 255]
    :param v: 亮度，范围 [0, 255]
    :return: Web 格式 HSV 颜色。三元组 `(h, s, v)`，范围分别为 (0-360, 0-100, 0-100)。
    """
    h = round(h * 2)  # opencv 的色相范围是 0-180，转为 0-360
    s = round(s / 255 * 100)  # opencv 的饱和度范围是 0-255，转为 0-100
    v = round(v / 255 * 100)  # opencv 的亮度范围是 0-255，转为 0-100
    return (h, s, v)

def rgb_to_hsv(c: RgbColor) -> 'HsvColor':
    """
    将 RGB 颜色转换为 HSV 颜色。

    :param c: RGB 颜色。十六进制颜色字符串 `#RRGGBB` 或整数三元组 `(r, g, b)`。
    :return: Web 格式 HSV 颜色。三元组 `(h, s, v)`，范围分别为 (0-360, 0-100, 0-100)。
    """
    c = _unify_color(c)
    ret = colorsys.rgb_to_hsv(c[0] / 255, c[1] / 255, c[2] / 255)
    return (round(ret[0] * 360), round(ret[1] * 100), round(ret[2] * 100))

def hsv_to_rgb(c: HsvColor) -> 'RgbColor':
    """
    将 HSV 颜色转换为 RGB 颜色。

    :param c: Web 格式 HSV 颜色。三元组 `(h, s, v)`，范围分别为 (0-360, 0-100, 0-100)。
    :return: RGB 颜色。整数三元组 `(r, g, b)`。
    """
    ret = colorsys.hsv_to_rgb(c[0] / 360, c[1] / 100, c[2] / 100)
    return (round(ret[0] * 255), round(ret[1] * 255), round(ret[2] * 255))

def _unify_color(color: RgbColor) -> RgbColorTuple:
    if isinstance(color, str):
        if not color.startswith('#'):
            raise ValueError('Hex color string must start with #')
        color = color[1:]  # 去掉#
        if len(color) != 6:
            raise ValueError('Hex color string must be 6 digits')
        r = int(color[0:2], 16)
        g = int(color[2:4], 16) 
        b = int(color[4:6], 16)
        return (r, g, b)
    elif (
        isinstance(color, tuple)
        and len(color) == 3
        and all(isinstance(c, int) for c in color)
        and all(0 <= c <= 255 for c in color)
    ):
        return color
    else:
        raise ValueError('Invalid color format')

def in_range(color: RgbColor, range: tuple[HsvColor, HsvColor]) -> bool:
    """
    判断颜色是否在范围内。

    :param color: RGB 颜色。
    :param range: Web HSV 颜色范围。
    """
    h, s, v = rgb_to_hsv(color)
    h1, s1, v1 = range[0]
    h2, s2, v2 = range[1]
    return h1 <= h <= h2 and s1 <= s <= s2 and v1 <= v <= v2

def find(
    image: MatLike | str,
    color: RgbColor,
    *,
    rect: Rect | None = None,
    threshold: float = 0.95,
    method: Literal['rgb_dist'] = 'rgb_dist',
) -> tuple[int, int] | None:
    """
    在图像中查找指定颜色的点。

    :param image: 
        图像。可以是 MatLike 或图像文件路径。
        注意如果参数为 MatLike，则颜色格式必须为 BGR，而不是 RGB。
    :param color: 颜色。可以是整数三元组 `(r, g, b)` 或十六进制颜色字符串 `#RRGGBB`。
    :param rect: 查找范围。如果为 None，则在整个图像中查找。
    :param threshold: 阈值，越大表示越相似，1 表示完全相似。默认为 0.95。
    :param method: 比较算法。默认为 'rgb_dist'，且目前也只有这个方法。

    ## 比较算法
    * rgb_dist:
        计算图片中每个点的颜色到目标颜色的欧氏距离，并以 442 为最大值归一化到 0-1 之间。
    """
    ret = None
    ret_similarity = 0
    found_color = None
    color = _unify_color(color)
    image = unify_image(image)
    image_rgb = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

    # 将目标颜色转换为HSL
    r, g, b = color
    target_rgb = np.array([[[r, g, b]]], dtype=np.uint8)
    target_hls = cv2.cvtColor(target_rgb, cv2.COLOR_RGB2HLS)[0,0]
    target_h, target_l, target_s = target_hls

    # 将图像转换为HSL
    image_hls = cv2.cvtColor(image, cv2.COLOR_BGR2HLS).astype(np.float32)
    
    # 计算HSL空间中的距离
    # H通道需要特殊处理,因为它是环形的(0和180是相邻的)
    h_diff = np.minimum(
        np.abs(image_hls[:,:,0] - target_h),
        180 - np.abs(image_hls[:,:,0] - target_h)
    )
    l_diff = np.abs(image_hls[:,:,1] - target_l)
    s_diff = np.abs(image_hls[:,:,2] - target_s)
    
    # 归一化距离(H:0-180, L:0-255, S:0-255)
    h_diff = h_diff / 90  # 最大差值180/2
    l_diff = l_diff / 255
    s_diff = s_diff / 255
    
    # 计算加权距离
    dist = np.sqrt((h_diff * 2)**2 + l_diff**2 + s_diff**2) / np.sqrt(6)
    
    # 寻找结果
    matches: np.ndarray = dist <= (1 - threshold)
    # 只在rect范围内搜索
    if rect is not None:
        x, y, w, h = rect
        search_area = matches[y:y+h, x:x+w]
        if search_area.any():
            # 在裁剪区域中找到最小距离的点
            local_dist = dist[y:y+h, x:x+w]
            local_dist[~search_area] = float('inf')
            min_y, min_x = np.unravel_index(np.argmin(local_dist), local_dist.shape)
            # 转换回原图坐标
            ret = (int(x + min_x), int(y + min_y))
            ret_similarity = 1 - local_dist[min_y, min_x]
            found_color = tuple(image_rgb[y+min_y, x+min_x])
    # 在全图中找到最小距离的点
    else:
        if matches.any():
            dist[~matches] = float('inf')
            min_y, min_x = np.unravel_index(np.argmin(dist), dist.shape)
            ret = (int(min_x), int(min_y))
            ret_similarity = 1 - dist[min_y, min_x]
            found_color = tuple(image_rgb[min_y, min_x])
    # 调试输出
    if debug.enabled:
        result_image = image.copy()
        # 绘制结果点
        if ret is not None:
            x, y = ret
            # 蓝色圈出结果点
            cv2.rectangle(result_image, 
                (max(0, x-20), max(0, y-20)),
                (min(result_image.shape[1], x+20), min(result_image.shape[0], y+20)),
                (255, 0, 0), 2)
        # 绘制搜索范围
        if rect is not None:
            x, y, w, h = rect
            # 红色圈出rect
            cv2.rectangle(result_image, (x, y), (x+w, y+h), (0, 0, 255), 2)
        debug_result(
            'find_rgb',
            [result_image, image],
            f'target={debug_color(color)}\n'
            f'rect={rect}\n'
            f'result={ret}\n'
            f'similarity={ret_similarity}\n'
            f'found_color={debug_color(found_color)}\n'
            '(Red rect for search area, blue rect for result area)'
        )
    return ret

def color_distance_map(
    image: MatLike | str,
    color: RgbColor,
    *,
    rect: Rect | None = None,
) -> np.ndarray:
    """
    计算图像中每个像素点到目标颜色的HSL距离，并返回归一化后的距离矩阵。
    
    :param image: 图像。可以是 MatLike 或图像文件路径。
    :param color: 目标颜色。可以是整数三元组 `(r, g, b)` 或十六进制颜色字符串 `#RRGGBB`。
    :param rect: 计算范围。如果为 None，则在整个图像中计算。
    :return: 归一化后的距离矩阵，范围 [0, 1]，0 表示完全匹配，1 表示完全不匹配。
    """
    # 统一颜色格式
    color = _unify_color(color)
    # 统一图像格式
    image = unify_image(image)
    
    # # 如果指定了rect，裁剪图像
    if rect is not None:
        x, y, w, h = rect
        image = image[y:y+h, x:x+w]

    # 将图像转换为HLS格式
    image_hls = cv2.cvtColor(image, cv2.COLOR_BGR2HLS).astype(np.float32)
    
    # 将目标颜色转换为HSL
    r, g, b = color
    target_rgb = np.array([[[r, g, b]]], dtype=np.uint8)
    target_hls = cv2.cvtColor(target_rgb, cv2.COLOR_RGB2HLS)[0,0]
    target_h, target_l, target_s = target_hls
    
    # 计算HSL空间中的距离
    # H通道需要特殊处理,因为它是环形的(0和180是相邻的)
    h_diff = np.minimum(
        np.abs(image_hls[:,:,0] - target_h),
        180 - np.abs(image_hls[:,:,0] - target_h)
    )
    l_diff = np.abs(image_hls[:,:,1] - target_l)
    s_diff = np.abs(image_hls[:,:,2] - target_s)
    
    # 归一化距离(H:0-180, L:0-255, S:0-255)
    h_diff = h_diff / 90  # 最大差值180/2
    l_diff = l_diff / 255
    s_diff = s_diff / 255
    
    # 计算加权距离
    dist = np.sqrt((h_diff * 2)**2 + l_diff**2 + s_diff**2) / np.sqrt(6)
    return dist

def _rect_intersection(rect1: Rect, rect2: Rect) -> Rect | None:
    """
    计算两个矩形的交集区域。
    
    :param rect1: 第一个矩形 (x, y, w, h)
    :param rect2: 第二个矩形 (x, y, w, h)
    :return: 交集矩形 (x, y, w, h)，如果没有交集则返回 None
    """
    x1 = max(rect1[0], rect2[0])
    y1 = max(rect1[1], rect2[1])
    x2 = min(rect1[0] + rect1[2], rect2[0] + rect2[2])
    y2 = min(rect1[1] + rect1[3], rect2[1] + rect2[3])
    
    if x1 >= x2 or y1 >= y2:
        return None
    
    return (x1, y1, x2 - x1, y2 - y1)

def find_all(
    image: MatLike | str,
    color: RgbColor,
    *,
    rect: Rect | None = None,
    threshold: float = 0.95,
    method: Literal['rgb_dist'] = 'rgb_dist',
    filter_method: Literal['point', 'contour'] = 'contour',
    max_results: int | None = None,
) -> list[FindColorPointResult]:
    """
    在图像中查找所有符合指定颜色的点。

    :param image: 
        图像。可以是 MatLike 或图像文件路径。
        注意如果参数为 MatLike，则颜色格式必须为 BGR，而不是 RGB。
    :param color: 颜色。可以是整数三元组 `(r, g, b)` 或十六进制颜色字符串 `#RRGGBB`。
    :param rect: 查找范围。如果为 None，则在整个图像中查找。
    :param threshold: 阈值，越大表示越相似，1 表示完全相似。默认为 0.95。
    :param method: 比较算法。默认为 'rgb_dist'，且目前也只有这个方法。
    :param filter_method: 查找方法。

        * point：按点查找结果
        * contour：按轮廓查找结果。也就是如果很多个符合要求的点连在一起，会被当做一个整体返回。
    :param max_results: 最大返回结果数量。如果为 None，则返回所有结果。
    :return: 结果列表。
    """
    # 计算距离矩阵
    dist = color_distance_map(image, color)
    # 筛选满足要求的点，二值化
    binary = np.where(dist <= (1 - threshold), 255, 0).astype(np.uint8)

    def filter_by_point(binary: np.ndarray, target_color: RgbColor, dist: np.ndarray, rect: Rect | None = None, max_results: int | None = None) -> list[FindColorPointResult]:
        results = []
        
        if rect is not None:
            x, y, w, h = rect
            search_area = binary[y:y+h, x:x+w]
            local_dist = dist[y:y+h, x:x+w]
            
            # 获取所有匹配点的坐标
            match_coords = np.where(search_area)
            for local_y, local_x in zip(*match_coords):
                confidence = 1 - local_dist[local_y, local_x]
                global_x = x + local_x
                global_y = y + local_y
                results.append(FindColorPointResult(
                    position=(int(global_x), int(global_y)),
                    confidence=float(confidence),
                    target_color=target_color
                ))
        else:
            # 获取所有匹配点的坐标
            match_coords = np.where(binary)
            for y, x in zip(*match_coords):
                confidence = 1 - dist[y, x]
                results.append(FindColorPointResult(
                    position=(int(x), int(y)),
                    confidence=float(confidence),
                    target_color=target_color
                ))

        # 按置信度排序
        results.sort(key=lambda x: x.confidence, reverse=True)
        
        # 限制结果数量
        if max_results is not None:
            results = results[:max_results]
            
        return results

    def filter_by_contour(binary: np.ndarray, target_color: RgbColor, dist: np.ndarray, rect: Rect | None = None, max_results: int | None = None) -> list[FindColorPointResult]:
        # 查找轮廓
        contours, _ = cv2.findContours(binary, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
        
        if not contours:
            return []
        
        results = []
        for contour in contours:
            # 获取轮廓的外界矩形
            contour_rect = cv2.boundingRect(contour)
            contour_rect = tuple(contour_rect)
            assert len(contour_rect) == 4
            
            # 如果指定了rect，计算轮廓外接矩形与rect的交集
            if rect is not None:
                intersection = _rect_intersection(contour_rect, rect)
                if intersection is None:
                    continue
                    
                x1, y1, w, h = intersection
                # 计算交集区域的中心点
                cx = x1 + w // 2
                cy = y1 + h // 2
                
                # 创建掩码，只保留当前轮廓内的区域
                mask = np.zeros_like(binary)
                cv2.drawContours(mask, [contour], 0, (255,), -1)
                
                # 只计算交集区域内的平均距离
                mask[y1:y1+h, x1:x1+w] = mask[y1:y1+h, x1:x1+w] & binary[y1:y1+h, x1:x1+w]
                avg_dist = np.mean(dist[y1:y1+h, x1:x1+w][mask[y1:y1+h, x1:x1+w] == 255])
            else:
                # 计算轮廓中心点
                cx = contour_rect[0] + contour_rect[2] // 2
                cy = contour_rect[1] + contour_rect[3] // 2
                
                # 创建掩码，只保留当前轮廓内的区域
                mask = np.zeros_like(binary)
                cv2.drawContours(mask, [contour], 0, (255,), -1)
                
                # 计算整个轮廓区域内的平均距离
                avg_dist = np.mean(dist[mask == 255])
            
            confidence = float(1 - avg_dist)
            
            results.append(FindColorPointResult(
                position=(cx, cy),
                confidence=confidence,
                target_color=target_color
            ))
        
        # 按置信度排序
        results.sort(key=lambda x: x.confidence, reverse=True)
        
        # 限制结果数量
        if max_results is not None:
            results = results[:max_results]
            
        return results

    # 根据filter_method选择过滤方法
    if filter_method == 'point':
        results = filter_by_point(binary, color, dist, rect, max_results)
    else:  # filter_method == 'contour'
        results = filter_by_contour(binary, color, dist, rect, max_results)

    # 调试输出
    if debug.enabled:
        result_image = unify_image(image).copy()
        # 绘制所有结果点
        for result in results:
            x, y = result.position
            cv2.rectangle(result_image, 
                (max(0, x-10), max(0, y-10)),
                (min(result_image.shape[1], x+10), min(result_image.shape[0], y+10)),
                (255, 0, 0), 1)
        # 绘制搜索范围
        if rect is not None:
            x, y, w, h = rect
            cv2.rectangle(result_image, (x, y), (x+w, y+h), (0, 0, 255), 2)
        
        debug_result(
            'find_rgb_many',
            [result_image, unify_image(image)],
            f'target={debug_color(color)}\n'
            f'rect={rect}\n'
            f'found {len(results)} points\n'
            f'threshold={threshold}\n'
            f'filter_method={filter_method}\n'
            '(Red rect for search area, blue rects for result areas)'
        )

    return results

# https://stackoverflow.com/questions/43111029/how-to-find-the-average-colour-of-an-image-in-python-with-opencv
def dominant_color(
    image: MatLike | str,
    count: int = 1,
    *,
    rect: Rect | None = None,
) -> list[RgbColorStr]:
    """
    提取图像的主色调。

    :param image:
        图像。可以是 MatLike 或图像文件路径。
        如果是 MatLike，则颜色格式必须为 BGR。
    :param count: 提取的颜色数量。默认为 1。
    :param rect: 提取范围。如果为 None，则在整个图像中提取。
    """
    # 载入/裁剪图像
    img = unify_image(image)
    if rect is not None:
        x, y, w, h = rect
        img = img[y:y+h, x:x+w]
    
    pixels = np.float32(img.reshape(-1, 3))

    n_colors = count
    criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 200, .1)
    flags = cv2.KMEANS_RANDOM_CENTERS

    _, labels, palette = cv2.kmeans(pixels, n_colors, None, criteria, 10, flags) # type: ignore
    _, counts = np.unique(labels, return_counts=True)

    # 将颜色按出现次数排序
    indices = (-counts).argsort()
    dominant = palette[indices]
    
    # 转换为 RGB 格式并转为 16 进制颜色代码
    result: list[RgbColorStr] = []
    for i in range(min(n_colors, len(dominant))):
        color = dominant[i]
        # BGR -> RGB
        rgb = tuple(map(int, color[::-1]))
        hex_color = f'#{rgb[0]:02x}{rgb[1]:02x}{rgb[2]:02x}'
        result.append(hex_color)
    
    if debug.enabled:
        origin_image = unify_image(image)
        result_image = origin_image.copy()
        if rect is not None:
            x, y, w, h = rect
            cv2.rectangle(result_image, (x, y), (x + w, y + h), (0, 0, 255), 2)
        debug_result(
            'color.dominant_color',
            [result_image, origin_image],
            f'arguments:\n \tcount={count}\n \trect={rect}\n'
            f'result={", ".join(map(debug_color, result))}'
        )

    return result

if __name__ == '__main__':
    img = cv2.imread('tests/images/ui/commu_fast_forward_enabled.png')
    colors = dominant_color(img, 3)
    print("主调颜色:")
    for i, color in enumerate(colors, 1):
        print(f"{i}. {color}")
        # 创建一个纯色图像来展示颜色
        color_block = np.full((100, 100, 3), _unify_color(color)[::-1], dtype=np.uint8)
        cv2.imshow(f"Color {i}", color_block)
    cv2.waitKey(0)
    cv2.destroyAllWindows()



WHATS_NEW = """
# 更新日志
## kaa
### v2025.3.22.0
脚本：
* [新增] 支持处理培育中技能卡自选删除（#f026f5a）
* [新增] 支持 Windows 平台的启动游戏（#04ddac5）
* [新增] 支持跳过培育结束的演出（#cc73188）
* [新增] 配置文件中引入 windows 截图方式（#0f92c6f）
* [修复] 修复某些情况下无法检测到进行中培育（#9e764c7）
* [修复] 修复某些情况下会跳过未读交流时会发生误触（#46c7354）
* [修复] 修复某些情况下竞赛时会卡在 SKIP 按钮上（#69a5987）
* [修复] 修复进入竞赛时无法跳过赛季奖励（#8356fa1）
* [重构] 将 AutoHotKey.exe 移动到 ksaa-res 包内（#1c063f3）
* [重构] 将培育中 acquisitions 函数所有 OCR 改为模板匹配（#6e336d9）
* [重构] 重构 toolbar_home，提取 toolbar_menu（#425e998）

框架：
* [新增] 将 kotonebot.client.Device 类分为了 AndroidDevice 和 WindowsDevice（#abe9b0a）
* [新增] 引入对 Windows 的支持（#d38bb16）
* [新增] 新增图像预处理器（#b831e9e）
* [修复] 修复首次启动时创建的配置版本号不正确（#aab6fc3）

开发工具：
* [新增] ImageAnnotation 中支持修改现有定义的标注类型（#7bd921b）

其他：
* [其他] 移除无用 import（#f902315）
* [其他] 添加针对 DMM/PC 平台的 `启动（Windows）.bat` 文件（#a86004b）
* [其他] 增加 Windows 平台依赖（#45a9838）
* [其他] 修复 just package 命令未不会自动创建环境的问题（#d08af8f）

### v2025.3.15.2
脚本：
* [修复] 修复培育开始时部分偶像会卡在偶像选择页面的问题（#6429826b）

框架：
* [修复] 修复雷电模拟器的自启动失败问题 & 调整相关UI（#972a4789）

### v2025.3.15.1
脚本：
* [修复] 再次修复对于某些偶像，培育开始时会卡在偶像选择页（#80be65d）

### v2025.3.15.0
脚本：
* [修复] 修复没有配置文件时，升级配置会失败的问题（#7f00eef）
* [修复] 修复扭蛋在慢速设备下的两个问题（#8f7650e）
* [修复] 修复了社团奖励领取后，ocr过早检测的问题（#54787f4）
* [修复] 修复了某些情况下社团奖励领取失效（#76475f4）
* [修复] 修复进入竞赛时有时会发生误触（#facac04）
* [修复] 修复对于某些偶像，培育开始时会卡在偶像选择页（#f13244a）
* [新增] 新增扭蛋机功能，支持任意次数的任意类型扭蛋（#fe3dd29）
* [新增] 新增升级一张低等级支援卡的任务（#a0216ba）
* [新增] 领取社团奖励（笔记请求）和送礼物（#e0cf155）
* [新增] 在配置项中加入跟踪功能的启停（#9949f5b）
* [新增] 竞赛支持选择第几个挑战者(1/2/3)（#6dc4cee）
* [新增] 支持kuyo模拟器启动（#b7805a2）
* [新增] 新增推荐卡检测严格模式（#ea3f931）
* [新增] 新增跟踪功能 & 支持跟踪推荐卡检测（#9a45a9b）
* [重构] 用ocr来区分社团奖励中的笔记请求是否结束（#f0077e3）
* [重构] 将 PIdol 改为 IntEnum（#724ad9f）
* [重构] 用资源文件和HintPoint代替club_reward中硬编码的坐标（#818acb6）
* [重构] 把kuyo启动作为start_game的一个子选项 & 添加包名自定义配置（#179cdfe）
* [重构] 合并 start_game 和 start_kuyo_and_game（#dd225ef）

界面：
* [修复] 修复长多选 Dropdown 在选择一个选项后滚动会自动重置的问题（#3283f15）
* [新增] 改进错误报告上传功能（#252cccf）

框架：
* [新增] create_device 时判断连接结果（#05e471a）
* [新增] 调试图片移除信息改为 VERBOSE 等级（#4afed09）
* [新增] MockDevice 新增支持 mock 图片数据（#1409255）

其他：
* [文档] 修改 README 路线图（#c4affe9）
* [文档] 纠正 REAMDE 中 アノマリー 拼写错误（#7d16bd6）
* [文档] 更新 README（#a140149）
* [其他] 从 git tags 中自动提取版本号（#64ae6ee）
* [其他] 为配置备份文件设置 gitignore（#392c8ab）
* [其他] XcantloadX/kotones-auto-assistant into main（#759e8c3）
* [其他] 将expect_wait前面的if not删去（#0b3e55e）
* [其他] 新增了pnpm-lock，在pnpm包管理器下确保依赖一致（#63e0772）
* [其他] 移除了club_reward.py中一些无用的import（#7dca24a）

### v2025.3.9.4
脚本：
* [修复] 修复培育中卡在考试前的问题

### v2025.3.9.3
其他：
* 降级 numpy<2.0，修复“A module that was compiled using NumPy 1.x cannot be run in NumPy 2.0.0 as it may crash.”问题

### v2025.3.9.1、v2025.3.9.2
其他：
* 降级 onnxruntime==1.14.0，修复“ImportError: DLL load failed while importing onnxruntime_pybind11_state”问题

### v2025.3.9.0
脚本：
* [修复] 修复部分情况下工作分配偶像选择页面报错（#a35e6c3）
* [修复] 修复培育中强化技能卡时有几率卡在确认按钮上（#221721a）
* [新增] 支持指定培育中行动优先级（#fc8c456）
* [新增] 支持处理培育结束时的未读交流（#452ba67）
* [新增] 培育中支持优先选择 SP 课程（#ee3808c）
* [新增] 培育中文化课（授業）行动支持处理自习事件（#47287fd）

界面：
* [新增] UI 上实时展示设备画面（#72a1a17）

框架：
* [修复] 修复调试模式下内存中截图数据不会释放的问题（#bead31a）
* [修复] 修复启动模拟器前不会自动检测是否已启动的问题（#71d75b7）
* [修复] 修复若软件处于中文目录，读取图片资源失败的问题（#a8aee32）
* [新增] 降低双击默认速度（#e63ad2d）
* [新增] 支持启动 kaa 前自动启动模拟器（#8ee51ba）
* [重构] 移动 grayscaled 和 unify_image 函数的位置（#500e11d）
* [重构] 移动 util 模块位置（#0d3e5de）
* [重构] 移除原有 kotonebot.run 模块中函数，改用 Kotonebot 类（#508d81c）

开发工具：
* [修复] 修复写出 dumps 文件时的编码问题（#d5c64df）
* [新增] 优化 DumpViewer 调用堆栈部分与时间部分的显示（#41d4c4c）
* [新增] ImageAnnotation 新增 description 字段（#425057d）
* [新增] DumpViewer 新增滚动锁定按钮（#79927a7）
* [新增] DumpViewer 两次结果之间新增时间差显示（#e3583f2）
* [重构] 将调试 WS 消息体提取为 pydantic Model（#abf7fbc）

其他：
* [其他] R.py 中的 HintBox 类型的注释也加入裁剪后图片的展示（#7907e44）

### v2025.3.1.0
脚本：
* [新增] 新增支持指定回忆编成编号（#6482962）
* [新增] 优化培育与日常流程，修复一些 bug（#145f97f）
* [新增] 支持处理外出行动 & 优化部分流程（#03fdfaa）
* [新增] 新增支持处理技能卡强化选择事件（#133f084）
* [新增] 培育中支持识别睡意 T 卡（眠気）（#5a68500）
* [修复] 修复课程中无法处理当前回合无剩余手牌的情况（#bfb574c）
* [修复] 修复课程中遇到四/五张手牌时会卡住的问题（#ae22253）
* [修复] 修复某些情况下会卡在培育开始第二步（#73dc4d8）
* [修复] 修复某些情况下会卡在工作分配偶像选择页面（#2ff655f）

界面：
* [新增] UI 上新增更新日志 Tab（#07be7b3）
* [新增] 新增一键上传日志报告功能（#62d5805）

框架：
* [新增] device.click() 方法新增支持传入 tuple[int, int]（#f146038）
* [新增] adb_raw 截图方法加入超时自动重试（#08f382c）
* [新增] 为 device.click() 方法添加可视调试信息（#be37164）
* [新增] 为 adb_raw 截图方式的截图线程加入自动重试（#9b25098）
* [修复] 修复了 device.click() 调试信息在没有截图数据时会报错（#f0a3baa）
* [修复] 修复写入调试数据前不会自动清空上次数据的问题（#80e04ee）

其他：
* [新增] ImageAnntation 页面与 Python 代码引入 HintPoint 支持（#949e28d）
* [修复] 修复某些情况下会卡在领取任务奖励处（#f80660e）
* [单测] 移除无用单元测试（#54f9b03）

### v2025.2.23.0
修复：
* 返回首页时无法处理弹出某些弹窗
* 培育中，某些情况下会卡在饮料领取弹窗
* 培育中，某些情况下会在饮料领取时报错

新增：
* 支持处理游戏本体更新弹窗
* 培育中，支持处理技能卡移除事件
* 培育结束时，支持处理“偶像强化月”活动的新纪录弹窗
* UI 标题上新增版本号显示

### v2025.2.20.1
修复：
* ADB 设备连接失败的问题

### v2025.2.20.0
初始版本。
"""


import logging
from typing import cast

LEVEL_VERBOSE = 5

def enable_verbose():
    logging.addLevelName(LEVEL_VERBOSE, 'VERBOSE')

class VerboseLogger(logging.Logger):
    def verbose(self, msg, *args, **kwargs):
        if self.isEnabledFor(LEVEL_VERBOSE):
            self._log(LEVEL_VERBOSE, msg, args, **kwargs)

def getLogger(name: str | None = None) -> VerboseLogger:
    logger = logging.getLogger(name)
    if not isinstance(logger, VerboseLogger):
        logger.__class__ = VerboseLogger
    return cast(VerboseLogger, logger)


import time
from typing_extensions import override

import cv2

from kotonebot.client.device import Device
from kotonebot import sleep

class Video:
    def __init__(self, path: str, fps: int):
        self.path = path
        self.fps = fps
        self.paused = False
        """是否暂停"""
        self.__cap = cv2.VideoCapture(path)
        self.__last_frame = None
        self.__last_time = 0

    def __iter__(self):
        return self

    def __next__(self):
        if self.paused:
            return self.__last_frame
        ret, frame = self.__cap.read()
        if not ret:
            raise StopIteration
        self.__last_frame = frame
        self.__last_time = time.time()
        if self.__last_time - time.time() < 1 / self.fps:
            sleep(1 / self.fps)
        return frame

    def pause(self):
        self.paused = True

    def resume(self):
        self.paused = False

class MockDevice(Device):
    def __init__(
        self
    ):
        super().__init__()
        self.__video_stream = None
        self.__image = None
        self.__screen_size = None

    def load_video(self, path: str, fps: int):
        self.__video_stream = Video(path, fps)
        return self.__video_stream

    def load_image(self, path: str):
        self.__image = cv2.imread(path)
        return self.__image

    def set_screen_size(self, width: int, height: int):
        self.__screen_size = (width, height)

    @override
    def screenshot(self):
        if self.__image is not None:
            return self.__image
        elif self.__video_stream is not None:
            return next(self.__video_stream)
        else:
            raise RuntimeError('No video stream loaded')
        
    @property
    @override
    def screen_size(self):
        if self.__screen_size is not None:
            return self.__screen_size
        else:
            raise RuntimeError('No screen size set')


import requests
import os

def upload(file_path: str) -> str:
    url = 'https://tmpsend.com/upload'
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/133.0.0.0 Safari/537.36',
        'Referer': 'https://tmpsend.com/',
    }
    
    file_name = os.path.basename(file_path)
    file_size = os.path.getsize(file_path)
    
    # 第一次请求：添加文件信息
    files = {
        'action': (None, 'add'),
        'name': (None, file_name),
        'size': (None, str(file_size)),
        'file': (file_name, open(file_path, 'rb'))
    }
    
    response = requests.post(url, headers=headers, files=files)
    if response.status_code != 200:
        raise Exception(f"Upload failed with status code {response.status_code}")
    
    result = response.json()
    if result.get('hasError'):
        raise Exception(result.get('error'))
    
    file_id = result.get('id')
    if not file_id:
        raise Exception("Failed to get file ID")
    
    # 第二次请求：上传实际文件
    upload_files = {
        'action': (None, 'upload'),
        'id': (None, file_id),
        'name': (None, file_name),
        'size': (None, str(file_size)),
        'start': (None, '0'),
        'end': (None, str(file_size)),
        'data': (file_name, open(file_path, 'rb'), 'application/octet-stream')
    }
    
    upload_response = requests.post(url, headers=headers, files=upload_files)
    if upload_response.status_code != 200:
        raise Exception(f"File upload failed with status code {upload_response.status_code}")
    
    return 'https://tmpsend.com/' + file_id

if __name__ == "__main__":
    file_path = r"主题1.thmx"
    print(upload(file_path))




import logging
import pkgutil
import importlib
from typing_extensions import Self
from dataclasses import dataclass, field
import threading
import traceback
import os
import zipfile
import cv2
from datetime import datetime
import io
from typing import Any, Literal, Callable, Generic, TypeVar, ParamSpec

from kotonebot.backend.context import Task, Action
from kotonebot.backend.context import init_context, vars
from kotonebot.backend.context import task_registry, action_registry, current_callstack, Task, Action
from kotonebot.client.host.protocol import Instance
from kotonebot.ui import user

log_stream = io.StringIO()
stream_handler = logging.StreamHandler(log_stream)
stream_handler.setFormatter(logging.Formatter('[%(asctime)s] [%(levelname)s] [%(name)s] [%(filename)s:%(lineno)d] - %(message)s'))
logging.getLogger('kotonebot').addHandler(stream_handler)
logger = logging.getLogger(__name__)

@dataclass
class TaskStatus:
    task: Task
    status: Literal['pending', 'running', 'finished', 'error', 'cancelled']

@dataclass
class RunStatus:
    running: bool = False
    tasks: list[TaskStatus] = field(default_factory=list)
    current_task: Task | None = None
    callstack: list[Task | Action] = field(default_factory=list)

    def interrupt(self):
        vars.interrupted.set()

def _save_error_report(
    exception: Exception,
    *,
    path: str | None = None
) -> str:
    """
    保存错误报告

    :param path: 保存的路径。若为 `None`，则保存到 `./reports/{YY-MM-DD HH-MM-SS}.zip`。
    :return: 保存的路径
    """
    from kotonebot import device
    try:
        if path is None:
            path = f'./reports/{datetime.now().strftime("%Y-%m-%d %H-%M-%S")}.zip'
        exception_msg = '\n'.join(traceback.format_exception(exception))
        task_callstack = '\n'.join([f'{i+1}. name={task.name} priority={task.priority}' for i, task in enumerate(current_callstack)])
        screenshot = device.screenshot()
        logs = log_stream.getvalue()
        with open('config.json', 'r', encoding='utf-8') as f:
            config_content = f.read()

        if not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        with zipfile.ZipFile(path, 'w') as zipf:
            zipf.writestr('exception.txt', exception_msg)
            zipf.writestr('task_callstack.txt', task_callstack)
            zipf.writestr('screenshot.png', cv2.imencode('.png', screenshot)[1].tobytes())
            zipf.writestr('config.json', config_content)
            zipf.writestr('logs.txt', logs)
        return path
    except Exception as e:
        logger.exception(f'Failed to save error report:')
        return ''

# Modified from https://stackoverflow.com/questions/70982565/how-do-i-make-an-event-listener-with-decorators-in-python
Params = ParamSpec('Params')
Return = TypeVar('Return')
class Event(Generic[Params, Return]):
    def __init__(self):
        self.__listeners = []
    
    @property
    def on(self):
        def wrapper(func: Callable[Params, Return]):
            self.add_listener(func)
            return func
        return wrapper
    
    def add_listener(self, func: Callable[Params, Return]) -> None:
        if func in self.__listeners:
            return
        self.__listeners.append(func)
    
    def remove_listener(self, func: Callable[Params, Return]) -> None:
        if func not in self.__listeners:
            return
        self.__listeners.remove(func)
    
    def __iadd__(self, func: Callable[Params, Return]) -> Self:
        self.add_listener(func)
        return self

    def __isub__(self, func: Callable[Params, Return]) -> Self:
        self.remove_listener(func)
        return self

    def trigger(self, *args: Params.args, **kwargs: Params.kwargs) -> None:
        for func in self.__listeners:
            func(*args, **kwargs)

class KotoneBotEvents:
    def __init__(self):
        self.task_status_changed = Event[
            [Task, Literal['pending', 'running', 'finished', 'error', 'cancelled']], None
        ]()
        self.task_error = Event[
            [Task, Exception], None
        ]()
        self.finished = Event[[], None]()


class KotoneBot:
    def __init__(
        self,
        module: str,
        config_type: type = dict[str, Any],
        *,
        debug: bool = False,
        resume_on_error: bool = False,
        auto_save_error_report: bool = True,
    ):
        """
        初始化 KotoneBot。

        :param module: 主模块名。此模块及其所有子模块都会被载入。
        :param config_type: 配置类型。
        :param debug: 调试模式。
        :param resume_on_error: 在错误时是否恢复。
        :param auto_save_error_report: 是否自动保存错误报告。
        """
        self.module = module
        self.config_type = config_type
        # HACK: 硬编码
        self.current_config: int | str = 0
        self.debug = debug
        self.resume_on_error = resume_on_error
        self.auto_save_error_report = auto_save_error_report
        self.events = KotoneBotEvents()
        self.backend_instance: Instance | None = None

    def initialize(self):
        """
        初始化并载入所有任务和动作。
        """
        logger.info('Initializing tasks and actions...')
        logger.debug(f'Loading module: {self.module}')
        # 加载主模块
        importlib.import_module(self.module)

        # 加载所有子模块
        pkg = importlib.import_module(self.module)
        for loader, name, is_pkg in pkgutil.walk_packages(pkg.__path__, pkg.__name__ + '.'):
            logger.debug(f'Loading sub-module: {name}')
            try:
                importlib.import_module(name)
            except Exception as e:
                logger.error(f'Failed to load sub-module: {name}')
                logger.exception(f'Error: ')
        
        logger.info('Tasks and actions initialized.')
        logger.info(f'{len(task_registry)} task(s) and {len(action_registry)} action(s) loaded.')

    def check_backend(self):
        from kotonebot.client.host import create_custom
        from kotonebot.config.manager import load_config
        # HACK: 硬编码
        config = load_config('config.json', type=self.config_type)
        config = config.user_configs[0]
        logger.info('Checking backend...')
        if config.backend.type == 'custom' and config.backend.check_emulator:
            exe = config.backend.emulator_path
            if exe is None:
                user.error('「检查并启动模拟器」已开启但未配置「模拟器 exe 文件路径」。')
                raise ValueError('Emulator executable path is not set.')
            if not os.path.exists(exe):
                user.error('「模拟器 exe 文件路径」对应的文件不存在！请检查路径是否正确。')
                raise FileNotFoundError(f'Emulator executable not found: {exe}')
            self.backend_instance = create_custom(
                adb_ip=config.backend.adb_ip,
                adb_port=config.backend.adb_port,
                adb_emulator_name=config.backend.adb_emulator_name,
                exe_path=exe
            )
            if not self.backend_instance.running():
                logger.info('Starting custom backend...')
                self.backend_instance.start()
                logger.info('Waiting for custom backend to be available...')
                self.backend_instance.wait_available()
            else:
                logger.info('Custom backend "%s" already running.', self.backend_instance)

    def run(self, tasks: list[Task], *, by_priority: bool = True):
        """
        按优先级顺序运行所有任务。
        """
        self.check_backend()
        init_context(config_type=self.config_type)

        if by_priority:
            tasks = sorted(tasks, key=lambda x: x.priority, reverse=True)
        for task in tasks:
            self.events.task_status_changed.trigger(task, 'pending')

        for task in tasks:
            logger.info(f'Task started: {task.name}')
            self.events.task_status_changed.trigger(task, 'running')

            if self.debug:
                task.func()
            else:
                try:
                    task.func()
                    self.events.task_status_changed.trigger(task, 'finished')
                # 用户中止
                except KeyboardInterrupt as e:
                    logger.exception('Keyboard interrupt detected.')
                    for task1 in tasks[tasks.index(task):]:
                        self.events.task_status_changed.trigger(task1, 'cancelled')
                    vars.interrupted.clear()
                    break
                # 其他错误
                except Exception as e:
                    logger.error(f'Task failed: {task.name}')
                    logger.exception(f'Error: ')
                    report_path = None
                    if self.auto_save_error_report:
                        report_path = _save_error_report(e)
                    self.events.task_status_changed.trigger(task, 'error')
                    if not self.resume_on_error:
                        for task1 in tasks[tasks.index(task)+1:]:
                            self.events.task_status_changed.trigger(task1, 'cancelled')
                        break
            logger.info(f'Task finished: {task.name}')
        logger.info('All tasks finished.')
        self.events.finished.trigger()

    def run_all(self) -> None:
        return self.run(list(task_registry.values()), by_priority=True)

    def start_all(self) -> RunStatus:
        run_status = RunStatus(running=True)
        def _on_finished():
            run_status.running = False
            run_status.current_task = None
            run_status.callstack = []
            self.events.finished -= _on_finished
            self.events.task_status_changed -= _on_task_status_changed
        
        def _on_task_status_changed(task: Task, status: Literal['pending', 'running', 'finished', 'error', 'cancelled']):
            def _find(task: Task) -> TaskStatus:
                for task_status in run_status.tasks:
                    if task_status.task == task:
                        return task_status
                raise ValueError(f'Task {task.name} not found in run_status.tasks')
            if status == 'pending':
                run_status.tasks.append(TaskStatus(task=task, status='pending'))
            else:
                _find(task).status = status

        self.events.task_status_changed += _on_task_status_changed
        self.events.finished += _on_finished
        thread = threading.Thread(target=self.run_all)
        thread.start()
        return run_status
        



from kotonebot.tasks.common import sprite_path
from kotonebot.backend.core import Image, HintBox, HintPoint



class Common:
    
    ButtonClose = Image(path=sprite_path(r"1792b27f-b8ab-424f-8c74-307230373823.png"), name="button_close.png")

    ButtonCommuFastforward = Image(path=sprite_path(r"d03227bd-d726-402a-8a16-9639a14c3b7b.png"), name="button_commu_fastforward.png")

    ButtonCompletion = Image(path=sprite_path(r"49eec332-87b5-4e82-b54b-2c0abf98f029.png"), name="button_completion.png")

    ButtonConfirm = Image(path=sprite_path(r"b01539ab-6449-4732-a879-22df632dbbd6.png"), name="button_confirm.png")

    ButtonConfirmNoIcon = Image(path=sprite_path(r"c43e4598-e8b7-4d6c-b41d-9ea0c910fd06.png"), name="button_confirm_no_icon.png")

    ButtonContest = Image(path=sprite_path(r"7ea01c5f-5c01-420b-8e37-36835fa3571c.png"), name="button_contest.png")

    ButtonEnd = Image(path=sprite_path(r"b95a376e-645c-4ba5-991e-6644f9a74b13.png"), name="button_end.png")

    ButtonHome = Image(path=sprite_path(r"47fe7859-20e8-4a46-b92b-8bcf7bb9f2dc.png"), name="button_home.png")

    ButtonIconArrowShort = Image(path=sprite_path(r"f3656fcc-a150-42bc-824d-9d40633ee111.png"), name="button_icon_arrow_short.png")

    ButtonIconArrowShortDisabled = Image(path=sprite_path(r"fdc00ad3-97db-4494-a55b-a7e14fa9a756.png"), name="button_icon_arrow_short_disabled.png")

    ButtonIconCheckMark = Image(path=sprite_path(r"b64b8cca-338e-4fc5-999d-50ad21a4b2da.png"), name="button_icon_check_mark.png")

    ButtonIconClose = Image(path=sprite_path(r"3c7b2002-c3ca-43cc-9008-24d207f34588.png"), name="button_icon_close.png")

    ButtonIdol = Image(path=sprite_path(r"884d10e1-fd5a-4254-823b-256937e9c4a5.png"), name="button_idol.png")

    ButtonIdolSupportCard = Image(path=sprite_path(r"bc06cce3-1e95-4f9f-a583-f28b7c9160e5.png"), name="button_idol_support_card.png")

    ButtonNext = Image(path=sprite_path(r"f4ad0154-d131-44e4-ac27-a22f82e9ed08.png"), name="button_next.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"03098b77-1501-416f-a7f5-f70f73c9f89e.png"), name="button_next_no_icon.png")

    ButtonRetry = Image(path=sprite_path(r"99d6049d-8c2a-45f4-a459-aff5ccba2e49.png"), name="button_retry.png")

    ButtonSelect = Image(path=sprite_path(r"d20ad0e2-6161-4e67-b0fc-fb2aec2877ab.png"), name="button_select.png")

    ButtonStart = Image(path=sprite_path(r"f1a62fb7-382b-44d1-afe7-8026c2f85eed.png"), name="button_start.png")

    ButtonToolbarHome = Image(path=sprite_path(r"94e94dcc-2ae2-44d9-ac15-c6428b1feb0b.png"), name="button_toolbar_home.png")

    ButtonToolbarMenu = Image(path=sprite_path(r"1a9b749e-9994-4070-b837-84969f6e926b.png"), name="button_toolbar_menu.png")

    CheckboxUnchecked = Image(path=sprite_path(r"b03b1726-2680-4010-b2b5-ef786cff3709.png"), name="checkbox_unchecked.png")

    ButtonToolbarHomeBinary = Image(path=sprite_path(r"2f68e457-4491-4ebc-b436-e3e63db92e14.png"), name="底部工具栏 首页按钮 二值化图片")

    ButtonToolbarMenuBinary = Image(path=sprite_path(r"bf6dcf21-2ffc-441f-82f3-59319f2313b1.png"), name="底部工具栏 菜单按钮 二值化图片")

    TextGameUpdate = Image(path=sprite_path(r"140066ba-0926-4ba2-84ab-8e27c321c52a.png"), name="text_game_update.png")

    TextNetworkError = Image(path=sprite_path(r"cc2d7443-704d-4623-9b1c-da3341f3fa2e.png"), name="text_network_error.png")

    ButtonCommuSkip = Image(path=sprite_path(r"f1f21925-3e22-4dd1-b53b-bb52bcf26c2b.png"), name="跳过交流按钮")

    ButtonOK = Image(path=sprite_path(r"8424ecdd-8857-4764-9fd0-d4bfa440c128.png"), name="OK 按钮")

    ButtonSelect2 = Image(path=sprite_path(r"5ebcde3b-f0fd-4e5d-b3de-ada8f0b5e03b.png"), name="選択する")

    IconButtonCheck = Image(path=sprite_path(r"fad5eec2-5fd5-412f-9abb-987a3087dc54.png"), name="按钮✓图标")

    IconButtonCross = Image(path=sprite_path(r"bc7155ac-18c9-4335-9ec2-c8762d37a057.png"), name="按钮×图标")


    pass
class Daily:
    
    ButonLinkData = Image(path=sprite_path(r"45148efb-d0d3-44b1-a3fa-7d828593a84c.png"), name="buton_link_data.png")

    ButtonAssignmentPartial = Image(path=sprite_path(r"f5487e79-114f-4d39-bff5-365f31114e21.png"), name="button_assignment_partial.png")

    ButtonClaimAllNoIcon = Image(path=sprite_path(r"1c88a1ab-5904-435c-928c-6434465b491a.png"), name="button_claim_all_no_icon.png")

    ButtonClubCollectReward = Image(path=sprite_path(r"6d805420-15dc-4b8c-9b9b-eb6f9015db6f.png"), name="button_club_collect_reward.png")

    ButtonClubSendGift = Image(path=sprite_path(r"0340eebc-084e-4566-b7c3-3f175c523614.png"), name="button_club_send_gift.png")

    ButtonClubSendGiftNext = Image(path=sprite_path(r"a0c121d3-6aa1-4e07-9024-fef4e8d48d1f.png"), name="button_club_send_gift_next.png")

    ButtonContestChallenge = Image(path=sprite_path(r"813b659e-25dc-47de-a9fb-5423ca6664da.png"), name="button_contest_challenge.png")

    ButtonContestChallengeStart = Image(path=sprite_path(r"51970f82-1d78-49d8-a099-e381b143dbfb.png"), name="button_contest_challenge_start.png")

    ButtonContestRanking = Image(path=sprite_path(r"3961e7f6-2a2e-4fee-9f86-19efd1d14e62.png"), name="button_contest_ranking.png")

    ButtonContestStart = Image(path=sprite_path(r"e2d8ffdc-f406-4134-8f52-7f06d2e85c23.png"), name="button_contest_start.png")

    ButtonDailyShop = Image(path=sprite_path(r"ab3ce6a4-bd60-412f-a8bc-1107d77a2590.png"), name="button_daily_shop.png")

    ButtonHomeCurrent = Image(path=sprite_path(r"14a6adaa-a791-48eb-b396-588edc7bc65d.png"), name="button_home_current.png")

    ButtonIconPass = Image(path=sprite_path(r"bbc23322-d73f-4884-a9b7-8a773887d262.png"), name="button_icon_pass.png")

    ButtonIconSkip = Image(path=sprite_path(r"d8ae4988-257a-4807-945b-94fd8a72518f.png"), name="button_icon_skip.png")

    ButtonMission = Image(path=sprite_path(r"8ce04389-a253-4cb3-a558-730dc3df5f1c.png"), name="button_mission.png")

    ButtonPass = Image(path=sprite_path(r"9501779e-1aaf-456a-81f2-f68af8344c58.png"), name="button_pass.png")

    ButtonPassClaim = Image(path=sprite_path(r"b9d6d4c8-5bef-4ac5-98c3-5514bea7d68f.png"), name="button_pass_claim.png")

    ButtonPresentsPartial = Image(path=sprite_path(r"d4fe14e7-3cf4-4827-aef3-13db86469014.png"), name="button_presents_partial.png")

    ButtonProduce = Image(path=sprite_path(r"3b142a19-d497-412f-9212-3182f9ba3557.png"), name="button_produce.png")

    ButtonShop = Image(path=sprite_path(r"7d06a260-3873-4325-92c5-0e0760c4b399.png"), name="button_shop.png")

    ButtonShopCapsuleToys = Image(path=sprite_path(r"33b53d6a-05eb-40fb-9b80-11c95e5586d5.png"), name="button_shop_capsule_toys.png")

    ButtonShopCapsuleToysDraw = Image(path=sprite_path(r"1092a47a-73e3-40fd-abe3-8a72056c7b42.png"), name="button_shop_capsule_toys_draw.png")

    ButtonShopCountAdd = Image(path=sprite_path(r"ace18c2f-50bd-4d62-9004-447b0ef2bbb5.png"), name="button_shop_count_add.png")

    ButtonShopCountAddDisabled = Image(path=sprite_path(r"2cd5f2aa-561e-4483-9b1c-7bf9f0f6afdc.png"), name="button_shop_count_add_disabled.png")

    ButtonSupportCardUpgrade = Image(path=sprite_path(r"4624770e-2753-4ff2-ac5a-093d9e1f82df.png"), name="button_support_card_upgrade.png")

    IconTitleDailyShop = Image(path=sprite_path(r"e9ee330d-dfca-440e-8b8c-0a3b4e8c8730.png"), name="日常商店标题图标")

    BoxHomeAssignment = HintBox(x1=16, y1=642, x2=127, y2=752, source_resolution=(720, 1280))

    BoxHomeAP = HintBox(x1=291, y1=4, x2=500, y2=82, source_resolution=(720, 1280))

    BoxHomeJewel = HintBox(x1=500, y1=7, x2=703, y2=82, source_resolution=(720, 1280))

    BoxHomeActivelyFunds = HintBox(x1=11, y1=517, x2=137, y2=637, source_resolution=(720, 1280))

    IconAssignKouchou = Image(path=sprite_path(r"14d3756c-001f-441c-aa81-0f6e74fafcd5.png"), name="icon_assign_kouchou.png")

    IconAssignMiniLive = Image(path=sprite_path(r"92ec7491-a1a7-4b3d-95ea-50852eeedf92.png"), name="icon_assign_mini_live.png")

    IconAssignOnlineLive = Image(path=sprite_path(r"a89e1b07-4f22-4ebb-ae27-1a948cf65350.png"), name="icon_assign_online_live.png")

    IconAssignTitle = Image(path=sprite_path(r"b7437da9-98e8-4261-b3ad-5d6c7c6a7b45.png"), name="icon_assign_title.png")

    IconMenuClub = Image(path=sprite_path(r"1fb58cec-d478-41f8-b0a0-826901322500.png"), name="icon_menu_club.png")

    IconShopAp = Image(path=sprite_path(r"6f38e88d-eb40-4769-898d-6a951ebfef4f.png"), name="icon_shop_ap.png")

    IconShopMoney = Image(path=sprite_path(r"e09005f4-7476-41ea-853d-8903e880f7b2.png"), name="icon_shop_money.png")

    IconShopTitle = Image(path=sprite_path(r"596ed0e2-15c3-4856-a2ba-44e4bb8ebdb9.png"), name="icon_shop_title.png")

    IconTitleAssign = Image(path=sprite_path(r"6408e2c0-d730-4675-9eae-8fd82b20ac96.png"), name="icon_title_assign.png")

    IconTitlePass = Image(path=sprite_path(r"5179ebec-bc81-4b2e-9cfd-2a3354c0d91b.png"), name="icon_title_pass.png")

    BoxApkUpdateDialogTitle = HintBox(x1=26, y1=905, x2=342, y2=967, source_resolution=(720, 1280))

    ButtonAssignmentShortenTime = Image(path=sprite_path(r"1652f06a-5417-49ef-8949-4854772d9ab7.png"), name="工作页面 短缩 时间")

    class Club:
        
        NoteRequestHintBox = HintBox(x1=314, y1=1071, x2=450, y2=1099, source_resolution=(720, 1280))
    
    
        pass
    PointDissmissContestReward = HintPoint(x=604, y=178)

    BoxMissonTabs = HintBox(x1=11, y1=929, x2=703, y2=1030, source_resolution=(720, 1280))

    class CapsuleToys:
        
        NextPageStartPoint = HintPoint(x=360, y=1167)
    
        NextPageEndPoint = HintPoint(x=362, y=267)
    
        IconTitle = Image(path=sprite_path(r"2bd6fe88-99fa-443d-8e42-bb3de5881213.png"), name="日常 扭蛋 页面标题图标")
    
        SliderStartPoint = HintPoint(x=476, y=898)
    
        SliderEndPoint = HintPoint(x=230, y=898)
    
    
        pass
    class SupportCard:
        
        DragDownStartPoint = HintPoint(x=357, y=872)
    
        DragDownEndPoint = HintPoint(x=362, y=194)
    
        TargetSupportCard = HintPoint(x=138, y=432)
    
    
        pass
    TextActivityFundsMax = Image(path=sprite_path(r"a02eba41-6d2f-486e-a07c-2393b927858b.png"), name="text_activity_funds_max.png")

    TextAssignmentCompleted = Image(path=sprite_path(r"70a257e5-aa4d-4fc1-a436-7564d5e83cab.png"), name="text_assignment_completed.png")

    TextContest = Image(path=sprite_path(r"51124743-b57b-48af-b65f-e1704bf7bafa.png"), name="text_contest.png")

    TextContestLastOngoing = Image(path=sprite_path(r"2b466959-f482-4453-b1fa-ebc81a282d30.png"), name="text_contest_last_ongoing.png")

    TextContestNoMemory = Image(path=sprite_path(r"5994b286-e6a4-4c80-ae5b-9d358cac1f11.png"), name="text_contest_no_memory.png")

    TextContestOverallStats = Image(path=sprite_path(r"ff1eb1f1-284c-4f87-be3e-734d0df814b3.png"), name="text_contest_overall_stats.png")

    TextShopPurchased = Image(path=sprite_path(r"d437fd76-9416-4858-b381-6190a82c8fad.png"), name="text_shop_purchased.png")

    TextShopRecommended = Image(path=sprite_path(r"690aaeb1-a725-4f12-bc67-dfb80114d424.png"), name="text_shop_recommended.png")

    TextTabShopAp = Image(path=sprite_path(r"ad28c735-2259-4fdc-9e9c-099c35c7c410.png"), name="text_tab_shop_ap.png")


    pass
class Shop:
    
    ItemLessonNote = Image(path=sprite_path(r"0949c622-9067-4f0d-bac2-3f938a1d2ed2.png"), name="レッスンノート")

    ItemVeteranNote = Image(path=sprite_path(r"b2af59e9-60e3-4d97-8c72-c7ba092113a3.png"), name="ベテランノート")

    ItemSupportEnhancementPt = Image(path=sprite_path(r"835489e2-b29b-426c-b4c9-3bb9f8eb6195.png"), name="サポート強化Pt 支援强化Pt")

    ItemSenseNoteVocal = Image(path=sprite_path(r"c5b7d67e-7260-4f08-a0e9-4d31ce9bbecf.png"), name="センスノート（ボーカル）感性笔记（声乐）")

    ItemSenseNoteDance = Image(path=sprite_path(r"0f7d581d-cea3-4039-9205-732e4cd29293.png"), name="センスノート（ダンス）感性笔记（舞蹈）")

    ItemSenseNoteVisual = Image(path=sprite_path(r"d3cc3323-51af-4882-ae12-49e7384b746d.png"), name="センスノート（ビジュアル）感性笔记（形象）")

    ItemLogicNoteVocal = Image(path=sprite_path(r"a1df3af1-a3e7-4521-a085-e4dc3cd1cc57.png"), name="ロジックノート（ボーカル）理性笔记（声乐）")

    ItemLogicNoteDance = Image(path=sprite_path(r"a9fcaf04-0c1f-4b0f-bb5b-ede9da96180f.png"), name="ロジックノート（ダンス）理性笔记（舞蹈）")

    ItemLogicNoteVisual = Image(path=sprite_path(r"c3f536d6-a04a-4651-b3f9-dd2c22593f7f.png"), name="ロジックノート（ビジュアル）理性笔记（形象）")

    ItemAnomalyNoteVocal = Image(path=sprite_path(r"eef25cf9-afd0-43b1-b9c5-fbd997bd5fe0.png"), name="アノマリーノート（ボーカル）非凡笔记（声乐）")

    ItemAnomalyNoteDance = Image(path=sprite_path(r"df991b42-ed8e-4f2c-bf0c-aa7522f147b6.png"), name="アノマリーノート（ダンス）非凡笔记（舞蹈）")

    ItemAnomalyNoteVisual = Image(path=sprite_path(r"9340b854-025c-40da-9387-385d38433bef.png"), name="アノマリーノート（ビジュアル）非凡笔记（形象）")

    ItemRechallengeTicket = Image(path=sprite_path(r"ea1ba124-9cb3-4427-969a-bacd47e7d920.png"), name="再挑戦チケット 重新挑战券")

    ItemRecordKey = Image(path=sprite_path(r"1926f2f9-4bd7-48eb-9eba-28ec4efb0606.png"), name="記録の鍵  解锁交流的物品")

    class IdolPiece:
        
        倉本千奈_WonderScale = Image(path=sprite_path(r"6720b6e8-ae80-4cc0-a885-518efe12b707.png"), name="倉本千奈 WonderScale 碎片")
    
        篠泽广_光景 = Image(path=sprite_path(r"afa06fdc-a345-4384-b25d-b16540830256.png"), name="篠泽广 光景 碎片")
    
        紫云清夏_TameLieOneStep = Image(path=sprite_path(r"278b7d9c-707e-4392-9677-74574b5cdf42.png"), name="紫云清夏 Tame-Lie-One-Step 碎片")
    
        葛城リーリヤ_白線 = Image(path=sprite_path(r"74ff07b3-d91c-4579-80cd-379ed7020622.png"), name="葛城リーリヤ 白線 碎片")
    
        姫崎薪波_cIclumsy_trick  = Image(path=sprite_path(r"a7f5abf1-982f-4a55-8d41-3ad6f56798e0.png"), name="姫崎薪波 cIclumsy trick 碎片")
    
        花海咲季_FightingMyWay = Image(path=sprite_path(r"2bc00520-0afe-40e5-8743-d33fc6b2945a.png"), name="花海咲季 FightingMyWay 碎片")
    
        藤田ことね_世界一可愛い私 = Image(path=sprite_path(r"135ee57a-d30d-4ba8-83f0-9f1681a49ff7.png"), name="藤田ことね 世界一可愛い私 碎片")
    
        花海佑芽_TheRollingRiceball = Image(path=sprite_path(r"d15959bf-d07b-4f07-948a-c0aeaf17756a.png"), name="花海佑芽 The Rolling Riceball 碎片")
    
        月村手毬_LunaSayMaybe = Image(path=sprite_path(r"868b97a9-492e-4712-b47f-82b97495b019.png"), name="月村手毬 Luna say maybe 碎片")
    
    
        pass

    pass
class Produce:
    
    BoxProduceOngoing = HintBox(x1=179, y1=937, x2=551, y2=1091, source_resolution=(720, 1280))

    ButtonAutoSet = Image(path=sprite_path(r"7ca854b7-34e3-4e31-96eb-d7bb3f11a7d4.png"), name="button_auto_set.png")

    ButtonProduce = Image(path=sprite_path(r"1432980b-5f29-4baa-b73f-b186b82ef9db.png"), name="button_produce.png")

    ButtonProduceStart = Image(path=sprite_path(r"e409f8ec-479d-4cea-b0a8-7eb5bce2f4b2.png"), name="button_produce_start.png")

    ButtonRegular = Image(path=sprite_path(r"90a3b4d4-02c9-45ee-8d39-0f4a608cea3b.png"), name="button_regular.png")

    CheckboxIconNoteBoost = Image(path=sprite_path(r"f34c88b9-769c-4d9f-93c4-ead045233b3a.png"), name="checkbox_icon_note_boost.png")

    CheckboxIconSupportPtBoost = Image(path=sprite_path(r"4d1bf46b-3b4b-487e-885b-8e744cb34931.png"), name="checkbox_icon_support_pt_boost.png")

    BoxModeButtons = HintBox(x1=7, y1=818, x2=713, y2=996, source_resolution=(720, 1280))

    ButtonPIdolOverview = Image(path=sprite_path(r"e88c9ad1-ec37-4fcd-b086-862e1e7ce8fd.png"), name="Pアイドルー覧  P偶像列表展示")

    BoxStepIndicator = HintBox(x1=4, y1=11, x2=405, y2=99, source_resolution=(720, 1280))

    BoxSetCountIndicator = HintBox(x1=17, y1=671, x2=119, y2=707, source_resolution=(720, 1280))

    PointProduceNextSet = HintPoint(x=702, y=832)

    PointProducePrevSet = HintPoint(x=14, y=832)

    IconPIdolLevel = Image(path=sprite_path(r"30a6f399-6999-4f04-bb77-651e0214112f.png"), name="P偶像卡上的等级图标")

    KbIdolOverviewName = HintBox(x1=140, y1=16, x2=615, y2=97, source_resolution=(720, 1280))

    ButtonResume = Image(path=sprite_path(r"ccbcb114-7f73-43d1-904a-3a7ae660c531.png"), name="再開する")

    BoxResumeDialogProduceType = HintBox(x1=186, y1=491, x2=541, y2=550, source_resolution=(720, 1280))

    BoxResumeDialogIdolTitle = HintBox(x1=194, y1=794, x2=523, y2=840, source_resolution=(720, 1280))

    RadioTextSkipCommu = Image(path=sprite_path(r"8626458a-63e7-4288-96e3-cbd624e64b75.png"), name="radio_text_skip_commu.png")

    TextAPInsufficient = Image(path=sprite_path(r"4883c564-f950-4a29-9f5f-6f924123cd22.png"), name="培育 AP 不足提示弹窗 标题")

    ButtonRefillAP = Image(path=sprite_path(r"eaba6ebe-f0df-4918-aee5-ef4e3ffedcf0.png"), name="确认恢复AP按钮")

    ScreenshotNoEnoughAp2 = Image(path=sprite_path(r"8ab34547-96bd-459b-8d3d-a8bd38ff010d.png"), name="screenshot_no_enough_ap_2.png")

    ButtonSkipLive = Image(path=sprite_path(r"e5e84f9e-28da-4cf4-bcba-c9145fe39b07.png"), name="培育结束跳过演出按钮")

    TextSkipLiveDialogTitle = Image(path=sprite_path(r"b6b94f21-ef4b-4425-9c7e-ca2b574b0add.png"), name="跳过演出确认对话框标题")

    TextRentAvailable = Image(path=sprite_path(r"9d89731a-3e5d-4e1c-9c25-dbca52f8ada6.png"), name="text_rent_available.png")


    pass
class InPurodyuusu:
    
    A = Image(path=sprite_path(r"90c8fe73-a906-43c2-860d-f608c29fe9f1.png"), name="A.png")

    AcquireBtnDisabled = Image(path=sprite_path(r"c1e8886f-f3fb-4b05-80f3-2014bc403554.png"), name="acquire_btn_disabled.png")

    ButtonCancel = Image(path=sprite_path(r"349c643a-04cc-4e5c-a1d2-582e14f0a368.png"), name="button_cancel.png")

    ButtonComplete = Image(path=sprite_path(r"9f7b489c-d68e-46ee-82d3-8bed333a1efa.png"), name="button_complete.png")

    ButtonFinalPracticeDance = Image(path=sprite_path(r"9575ec77-9825-40d9-8803-a311843a372b.png"), name="button_final_practice_dance.png")

    ButtonFinalPracticeVisual = Image(path=sprite_path(r"976befa5-b285-45be-b04b-d3c0386f4699.png"), name="button_final_practice_visual.png")

    ButtonFinalPracticeVocal = Image(path=sprite_path(r"34f01361-adf1-4aab-8f2b-697a3c3bd17f.png"), name="button_final_practice_vocal.png")

    ButtonFollowNoIcon = Image(path=sprite_path(r"6dcd939f-e0b2-4346-888e-d92ad0fac266.png"), name="button_follow_no_icon.png")

    ButtonIconStudy = Image(path=sprite_path(r"0dbdce25-b2e6-44fa-b0a4-b82467a94ba7.png"), name="button_icon_study.png")

    ButtonIconStudyVisual = Image(path=sprite_path(r"0ef5eaa1-be89-4b70-a04c-5be3d7499c68.png"), name="button_icon_study_visual.png")

    ButtonLeave = Image(path=sprite_path(r"1e570f83-77dd-40b8-b2e5-516c09f8faa9.png"), name="button_leave.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"db140c0c-03d2-4e34-b8cf-a0c1c72d24ab.png"), name="button_next_no_icon.png")

    ButtonNotAcquire = Image(path=sprite_path(r"1634e858-6312-4c55-9734-4ecde427ba19.png"), name="button_not_acquire.png")

    ButtonRetry = Image(path=sprite_path(r"c1af1812-63d1-439d-b129-fca2106b5a62.png"), name="button_retry.png")

    ButtonTextActionOuting = Image(path=sprite_path(r"dfacb774-b6d8-45b5-9550-367fd3eeee19.png"), name="button_text_action_outing.png")

    ButtonTextAllowance = Image(path=sprite_path(r"4e0c6e03-262f-4c9e-8b5a-d00439376672.png"), name="button_text_allowance.png")

    ButtonTextConsult = Image(path=sprite_path(r"466ac973-10ec-4676-9f6f-1e2cd1fe97b4.png"), name="button_text_consult.png")

    IconClearBlue = Image(path=sprite_path(r"bee2adba-e36d-4c79-9061-be815164951a.png"), name="icon_clear_blue.png")

    IconTitleAllowance = Image(path=sprite_path(r"17568434-4ad5-46c5-b142-1b1f05614c08.png"), name="icon_title_allowance.png")

    IconTitleStudy = Image(path=sprite_path(r"284276f2-67b3-4d32-8358-516e136f6af6.png"), name="icon_title_study.png")

    LootboxSliverLock = Image(path=sprite_path(r"d18bf286-a85c-4866-83d5-3b3fa2e66fe3.png"), name="lootbox_sliver_lock.png")

    LootBoxSkillCard = Image(path=sprite_path(r"43f318d7-2f15-4b6f-863b-e2261bc97bbb.png"), name="loot_box_skill_card.png")

    M = Image(path=sprite_path(r"2c141963-1bc6-4c4f-9185-68a7d841d040.png"), name="M.png")

    BoxWeeksUntilExam = HintBox(x1=11, y1=8, x2=237, y2=196, source_resolution=(720, 1280))

    TextActionVocal = Image(path=sprite_path(r"d6b64759-26b7-45b1-bf8e-5c0d98611e0d.png"), name="Vo. レッスン")

    TextActionDance = Image(path=sprite_path(r"303cccc1-c674-4d3a-8c89-19ea729fdbef.png"), name="Da. レッスン")

    TextActionVisual = Image(path=sprite_path(r"cc8a495d-330d-447d-8a80-a8a6ecc409c5.png"), name="Vi. レッスン")

    BoxExamTop = HintBox(x1=5, y1=2, x2=712, y2=55, source_resolution=(720, 1280))

    PDrinkIcon = Image(path=sprite_path(r"ee84cfe3-f36d-42c4-a27c-033e8afeb077.png"), name="p_drink_icon.png")

    PItemIconColorful = Image(path=sprite_path(r"3eb3edc8-be33-4fcd-866a-0ae6fbbb932c.png"), name="p_item_icon_colorful.png")

    PSkillCardIconBlue = Image(path=sprite_path(r"ba402073-ef1b-42d0-8cfb-84ffefdc0c43.png"), name="p_skill_card_icon_blue.png")

    PSkillCardIconColorful = Image(path=sprite_path(r"a35bdd08-b319-47d0-be24-26dcf9789a04.png"), name="p_skill_card_icon_colorful.png")

    Rest = Image(path=sprite_path(r"f24a70f9-304e-4142-8002-be6baf523206.png"), name="rest.png")

    RestConfirmBtn = Image(path=sprite_path(r"2c750883-32c9-451b-817e-2fc1a190020c.png"), name="rest_confirm_btn.png")

    ButtonIconOuting = Image(path=sprite_path(r"8ded6c98-85ea-4858-a66d-4fc8caecb7c5.png"), name="行动按钮图标 外出（おでかけ）")

    TextGoalClearNext = Image(path=sprite_path(r"05890a1b-8764-4e9f-9d21-65d292c22e13.png"), name="培育目标达成 NEXT 文字")

    BoxLessonCards5_1 = HintBox(x1=16, y1=882, x2=208, y2=1136, source_resolution=(720, 1280))

    BoxNoSkillCard = HintBox(x1=180, y1=977, x2=529, y2=1026, source_resolution=(720, 1280))

    TitleIconOuting = Image(path=sprite_path(r"ee4e512b-4982-49b6-9c71-31984b58e1d0.png"), name="外出（おでかけ）页面 标题图标")

    TextPDrinkMaxConfirmTitle = Image(path=sprite_path(r"582d36c0-0916-4706-9833-4fbc026701f5.png"), name="P饮料溢出 不领取弹窗标题")

    IconTitleSkillCardRemoval = Image(path=sprite_path(r"bab6c393-692c-4681-ac0d-76c0d9dabea6.png"), name="技能卡自选删除 标题图标")

    ButtonRemove = Image(path=sprite_path(r"00551158-fee9-483f-b034-549139a96f58.png"), name="削除")

    TextPDrink = Image(path=sprite_path(r"8c179a21-be6f-4db8-a9b0-9afeb5c36b1c.png"), name="文本「Pドリンク」")

    BoxSelectPStuffComfirm = HintBox(x1=256, y1=1064, x2=478, y2=1128, source_resolution=(720, 1280))

    TextClaim = Image(path=sprite_path(r"c948f136-416f-447e-8152-54a1cd1d1329.png"), name="文字「受け取る」")

    TextPItem = Image(path=sprite_path(r"0c0627be-4a09-4450-a078-1858d3ace532.png"), name="文字「Pアイテム」")

    TextSkillCard = Image(path=sprite_path(r"d271a24f-efe8-424d-8fd5-f6b3756ba4ca.png"), name="文字「スキルカード」")

    BoxSkillCardAcquired = HintBox(x1=194, y1=712, x2=528, y2=765, source_resolution=(720, 1280))

    IconSkillCardEventBubble = Image(path=sprite_path(r"6b58d90d-2e5e-4b7f-bc01-941f2633de89.png"), name="技能卡事件气泡框图标")

    ScreenshotSkillCardEnhanceDialog = Image(path=sprite_path(r"249bb7b2-68cb-44b0-840a-855f76405177.png"), name="screenshot_skill_card_enhance_dialog.png")

    IconTitleSkillCardEnhance = Image(path=sprite_path(r"79abd239-5eed-4195-9fa8-d729daa874ca.png"), name="技能卡强化 标题 图标")

    ButtonEnhance = Image(path=sprite_path(r"da439e8c-3b74-4371-9657-0736d826c7d1.png"), name="技能卡 强化按钮")

    T = Image(path=sprite_path(r"16fbc93d-b294-4001-b4e9-ee2af181415f.png"), name="睡意卡字母 T（眠気）")

    IconSp = Image(path=sprite_path(r"d982d2b5-4bc0-4ae9-a516-f29c2848d64b.png"), name="SP 课程图标")

    BoxCommuEventButtonsArea = HintBox(x1=14, y1=412, x2=703, y2=1089, source_resolution=(720, 1280))

    TextSelfStudyVocal = Image(path=sprite_path(r"c78c38cc-7b61-4dc4-820d-0a5b684ef52e.png"), name="文化课事件 自习 声乐")

    TextSelfStudyDance = Image(path=sprite_path(r"83d0a033-466c-463a-bb8c-be0f2953e9b2.png"), name="文化课事件 自习 舞蹈")

    TextSelfStudyVisual = Image(path=sprite_path(r"4695f96b-c4f5-4bb6-a021-a13b6ceb2883.png"), name="文化课事件 自习 形象")

    TextAsariProduceEnd = Image(path=sprite_path(r"15ab4cd6-00ef-4fc2-b8b2-3f0fea8d121c.png"), name="text_asari_produce_end.png")

    TextButtonExamSkipTurn = Image(path=sprite_path(r"58d8977e-626a-4122-834e-be322e504ce8.png"), name="text_button_exam_skip_turn.png")

    TextClearUntil = Image(path=sprite_path(r"e4ed1a58-a1d3-406c-8433-9cb8a116870b.png"), name="text_clear_until.png")

    TextDance = Image(path=sprite_path(r"1fb62eeb-c8b5-44aa-a6d9-52e2efdeee7f.png"), name="text_dance.png")

    TextFinalProduceRating = Image(path=sprite_path(r"46708b7b-e2a5-4247-9050-933304c3e7d4.png"), name="text_final_produce_rating.png")

    TextOneWeekRemaining = Image(path=sprite_path(r"743d1b3b-a219-4240-89da-5dfd06330699.png"), name="text_one_week_remaining.png")

    TextPerfectUntil = Image(path=sprite_path(r"a6ee9746-3953-4867-babb-8397697cdcef.png"), name="text_perfect_until.png")

    TextPleaseSelectPDrink = Image(path=sprite_path(r"f625c4b0-1e06-4fd0-b047-080b2760e43e.png"), name="text_please_select_p_drink.png")

    TextPDiary = Image(path=sprite_path(r"e48a9693-13cd-463d-b4dc-d8cf63536c33.png"), name="text_p_diary.png")

    TextPDrinkMax = Image(path=sprite_path(r"0b15b72e-d3f0-414c-8c9b-3452e85d8a04.png"), name="text_p_drink_max.png")

    TextSenseiTipConsult = Image(path=sprite_path(r"a7780df5-74f7-420f-b845-0442454832df.png"), name="text_sensei_tip_consult.png")

    TextSenseiTipDance = Image(path=sprite_path(r"40ea9658-fcf6-44f9-b4f7-b5183f0c7089.png"), name="text_sensei_tip_dance.png")

    TextSenseiTipRest = Image(path=sprite_path(r"4055f351-9a78-4a01-bba7-081b380b0ef3.png"), name="text_sensei_tip_rest.png")

    TextSenseiTipVisual = Image(path=sprite_path(r"0cc78ac2-db49-4827-a258-63c9963201fe.png"), name="text_sensei_tip_visual.png")

    TextSenseiTipVocal = Image(path=sprite_path(r"175364b3-8f02-4c79-ae1c-cf1feb2908e9.png"), name="text_sensei_tip_vocal.png")

    TextSkipTurnDialog = Image(path=sprite_path(r"6265f543-48f4-43f3-9e80-f8bb184e3d57.png"), name="text_skip_turn_dialog.png")

    TextVisual = Image(path=sprite_path(r"f27bdd8b-6287-4278-b82d-3a9f6f0ef427.png"), name="text_visual.png")

    class Action:
        
        ActionStudy = Image(path=sprite_path(r"b29ebe51-1a79-4c4d-b6f5-9583602c0595.png"), name="action_study.png")
    
        PDorinkuBg = Image(path=sprite_path(r"26c68c62-8001-4238-95a9-b84faa735d65.png"), name="p_dorinku_bg.png")
    
        PDorinkuBgMask = Image(path=sprite_path(r"2e076e6e-0ed7-4f85-93ae-52bb07155ca1.png"), name="p_dorinku_bg_mask.png")
    
        PItem = Image(path=sprite_path(r"21bc6029-9370-4ffe-a0e8-120b899173a1.png"), name="p_item.png")
    
        VocalWhiteBg = Image(path=sprite_path(r"1ee95dc4-cfef-40df-b86e-534113bec38a.png"), name="vocal_white_bg.png")
    
    
        pass

    pass
class Kuyo:
    
    ButtonStartGame = Image(path=sprite_path(r"c5956d09-9154-4360-a280-443e8000cb85.png"), name="button_start_game.png")

    ButtonTab3Speedup = Image(path=sprite_path(r"0aebab91-020b-4bbd-baeb-920579a24531.png"), name="button_tab3_speedup.png")


    pass


import time
import asyncio
import inspect
import threading
import traceback
import subprocess
from io import StringIO
from pathlib import Path
from typing import Literal
from collections import deque
from contextlib import redirect_stdout

import cv2
import uvicorn
from thefuzz import fuzz
from pydantic import BaseModel
from fastapi.staticfiles import StaticFiles
from fastapi.responses import FileResponse, Response
from fastapi import FastAPI, WebSocket, HTTPException
from fastapi.middleware.cors import CORSMiddleware

import kotonebot
import kotonebot.backend
import kotonebot.backend.context
from kotonebot.backend.core import HintBox, Image
from ..context import manual_context
from . import vars as debug_vars
from .vars import WSImage, WSMessageData, WSMessage, WSCallstack

app = FastAPI()
app.add_middleware(CORSMiddleware, allow_origins=["*"], allow_credentials=True, allow_methods=["*"], allow_headers=["*"])

# 获取当前文件夹路径
CURRENT_DIR = Path(__file__).parent

APP_DIR = Path.cwd()

class File(BaseModel):
    name: str
    full_path: str
    type: Literal["file", "dir"]

@app.get("/api/read_file")
async def read_file(path: str):
    """读取文件内容"""
    try:
        # 确保路径在当前目录下
        full_path = (APP_DIR / path).resolve()
        if not Path(full_path).is_relative_to(APP_DIR):
            raise HTTPException(status_code=403, detail="Access denied")
        
        if not full_path.exists():
            raise HTTPException(status_code=404, detail="File not found")
        # 添加缓存控制头
        headers = {
            "Cache-Control": "public, max-age=3600",  # 缓存1小时
            "ETag": f'"{hash(full_path)}"'  # 使用full_path的哈希值作为ETag
        }
        return FileResponse(full_path, headers=headers)
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))

@app.get("/api/read_memory")
async def read_memory(key: str):
    """读取内存中的数据"""
    try:
        image = None
        if (image := debug_vars._read_image(key)) is not None:
            pass
        else:
            raise HTTPException(status_code=404, detail="Key not found")
        
        # 编码图片
        encode_params = [cv2.IMWRITE_PNG_COMPRESSION, 4]
        _, buffer = cv2.imencode('.png', image, encode_params)
        # 添加缓存控制头
        headers = {
            "Cache-Control": "public, max-age=3600",  # 缓存1小时
            "ETag": f'"{hash(key)}"'  # 使用key的哈希值作为ETag
        }
        return Response(
            buffer.tobytes(), 
            media_type="image/jpeg",
            headers=headers
        )
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))

@app.get("/api/screenshot")
def screenshot():
    from ..context import device
    img = device.screenshot()
    buff = cv2.imencode('.png', img)[1].tobytes()
    return Response(buff, media_type="image/png")

class RunCodeRequest(BaseModel):
    code: str

@app.post("/api/code/run")
async def run_code(request: RunCodeRequest):
    event = asyncio.Event()
    stdout = StringIO()
    code = f"from kotonebot import *\n" + request.code
    result = {}
    def _runner():
        nonlocal result
        from kotonebot.backend.context import vars as context_vars
        try:
            with manual_context():
                global_vars = dict(vars(kotonebot.backend.context))
                with redirect_stdout(stdout):
                    exec(code, global_vars)
            result = {"status": "ok", "result": stdout.getvalue()}
        except (Exception) as e:
            result = {"status": "error", "result": stdout.getvalue(), "message": str(e), "traceback": traceback.format_exc()}
        except KeyboardInterrupt as e:
            result = {"status": "error", "result": stdout.getvalue(), "message": str(e), "traceback": traceback.format_exc()}
        finally:
            context_vars.interrupted.clear()
        event.set()
    threading.Thread(target=_runner, daemon=True).start()
    await event.wait()
    return result

@app.get("/api/code/stop")
async def stop_code():
    from kotonebot.backend.context import vars
    vars.interrupted.set()
    while vars.interrupted.is_set():
        await asyncio.sleep(0.1)
    return {"status": "ok"}

@app.get("/api/fs/list_dir")
def list_dir(path: str) -> list[File]:
    result = []
    for item in Path(path).iterdir():
        result.append(File(
            name=item.name,
            full_path=str(item),
            type="file" if item.is_file() else "dir"
        ))
    return result

@app.get("/api/resources/autocomplete")
def autocomplete(class_path: str) -> list[str]:
    from kotonebot.tasks import R # HACK: hardcode
    class_names = class_path.split(".")[:-1]
    target_class = R
    # 定位到目标类
    for name in class_names:
        target_class = getattr(target_class, name, None)
        if target_class is None:
            return []
    # 获取目标类的所有属性
    attrs = [attr for attr in dir(target_class) if not attr.startswith("_")]
    filtered_attrs = []
    for attr in attrs:
        if inspect.isclass(getattr(target_class, attr)):
            filtered_attrs.append(attr)
        elif isinstance(getattr(target_class, attr), (Image, HintBox)):
            filtered_attrs.append(attr)  
    attrs = filtered_attrs
    # 排序
    attrs.sort(key=lambda x: fuzz.ratio(x, class_path), reverse=True)
    return attrs

@app.get("/api/ping")
async def ping():
    return {"status": "ok"}

message_queue = deque()
@app.websocket("/ws")
async def websocket_endpoint(websocket: WebSocket):
    await websocket.accept()
    try:
        while True:
            if len(message_queue) > 0:
                message = message_queue.popleft()
                await websocket.send_json(message)
            await asyncio.sleep(0.1)
    except:
        await websocket.close()

def send_ws_message(title: str, image: list[str], text: str = '', callstack: list[WSCallstack] = [], wait: bool = False):
    """发送 WebSocket 消息"""
    message = WSMessage(
        type="visual",
        data=WSMessageData(
            image=WSImage(type="memory", value=image),
            name=title,
            details=text,
            timestamp=int(time.time() * 1000),
            callstack=callstack
        )
    )
    message_queue.append(message.dict())
    if wait:
        while len(message_queue) > 0:
            time.sleep(0.3)


thread = None
def start_server():
    global thread
    def run_server():
        uvicorn.run(app, host="127.0.0.1", port=8000, log_level='critical' if debug_vars.debug.hide_server_log else None)
    if thread is None:
        thread = threading.Thread(target=run_server, daemon=True)
        thread.start()

def wait_message_all_done():
    global thread
    def _wait():
        while len(message_queue) > 0:
            time.sleep(0.1)
    if thread is not None:
        threading.Thread(target=_wait, daemon=True).start()

if __name__ == "__main__":
    from kotonebot.backend.context import init_context
    init_context()
    debug_vars.debug.hide_server_log = False
    process = subprocess.Popen(["pylsp", "--port", "5479", "--ws"])
    print("LSP started. PID=", process.pid)
    uvicorn.run(app, host="127.0.0.1", port=8000, log_level='critical' if debug_vars.debug.hide_server_log else None)
    process.kill()